/*
 * Copyright (C) 2001 Karsten Reincke <karsten.reincke@gnukose.org>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 * file <muo_convert.cc> version <#1.3.1#> of project <MrProjext>
 */   

/** \file muo_convert.cc
 *  \brief all definitions of and for the conversion of strings
 *
 *  \author karsten reincke
 */

#include <assert.h>
#include <iostream>
#include <string>
#include <cstdlib>
#include <cstdio>
#include "muo_convert.h"

using namespace std;


/* &&& (1) local preprocessor-defines &&&&&&&&&&&&&&&&&&&&&&& */

/* &&& (2) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&& (2) definitions of static global variables &&&&&&&&&&& */

/* &&& (3) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&&     methods for class &&& */

/* --- (3.A) - public ----------------------------------------- */
/* :-: (3.A.1) public constructors, inits & operators :-:-:-:-: */
/** 
 *  \brief general constructor for any type of Converter
 *
 *  intializes all member variables
 */
krmuo::Converter::Converter()
: mUnConvertedToken(""),
  mConvertedToken(""),
  mConvertedString(""),
  mUtf8String(""),
  mTokenMap(NULL)
{
  mUtf8Sign[0]=0;
}

/** 
 * \brief general deconstructor for any type of Converter
 */
krmuo::Converter::~Converter()
{
}

/* :-: (3.A.2) public getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (3.A.3) public setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (3.A.4) public others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/** 
 *  \brief realize a (default) decodation (valid for xml and html)
 *
 *  \param pUnDecodedString :- a row of undecoded entities
 *  \return a copy(!) of the internal result string
 *
 *  while any undecoded entity can be found
 *  decoded that entity and append the result
 *  ( normally a one-sign-string )
 *  to the internal result string of the conversion
 */
const string krmuo::Converter::decode(const string& pUnDecodedString)
{
  assert(mTokenMap);
  mUnConvertedStringEnd=pUnDecodedString.end();
  mUnConvertedStringIterator=pUnDecodedString.begin();
  mConvertedString="";
  unsigned char sign;
  while(getNextUnDecodedToken(mUnConvertedToken))
  {
    if (isDecodableEmbeddedUtf8Token(mUnConvertedToken,sign))
      mConvertedString+=sign;
    else if (mTokenMap->isDecodableToken(mUnConvertedToken))
      mConvertedString+=mTokenMap->decodeToken(mUnConvertedToken);
    else
      mConvertedString+=mUnConvertedToken;
  }
  return mConvertedString;
}

/** 
 *  \brief realize an encodation
 *
 *  \param pUnEncodedString :- a row of unencoded signs
 *  \return copy(!) of the internal result string
 *
 *  while any undecoded signs can be found
 *  encoded that sign and append the result (= entity(!))
 *  to the internal result string of the conversion
 */
const string krmuo::Converter::encode(const string& pUnEncodedString)
{
  assert(mTokenMap);
  mUnConvertedStringIterator=pUnEncodedString.begin();
  mConvertedString="";
  while(mUnConvertedStringIterator!=pUnEncodedString.end())
  {
    if (mTokenMap->isEncodableSign((unsigned char)*mUnConvertedStringIterator))
      mConvertedString+=mTokenMap->encodeSign((unsigned char)*mUnConvertedStringIterator);
    else if (isUtf8EncodableSign((unsigned char)*mUnConvertedStringIterator,mUtf8String))
      mConvertedString+=mUtf8String;
    else
      mConvertedString+=(unsigned char)*mUnConvertedStringIterator;
    mUnConvertedStringIterator++;
  }
  return mConvertedString;
}

/**
 *  \brief utf8-decodable (xml/html) tokens have the structure &#...;
 *  \param pUtf8Token :- token which shall be tested
 *  \param pSign :- reference to unsigned char into which the value will be stored
 *  \return true if token is utf8decodable and content of pSign is valid
 */
bool krmuo::Converter::isDecodableEmbeddedUtf8Token
( const string& pUtf8Token,
  unsigned char& pSign
)
{
  if (pUtf8Token.size()<3)
    return false;
    
  string::const_iterator tokenSign=pUtf8Token.begin(); 
  if (((unsigned char)* tokenSign)  != ((unsigned char)'&'))
    return false;

  tokenSign++;
  if (((unsigned char) *tokenSign) != ((unsigned char)'#'))
    return false;
               
  tokenSign++;
  // *tokensign == 'x' | 'X' | 'o' | 'O' | '1' ... '0'
    
  enum NumberStil { UNKNOWN, HEX, OCTAL, DECIMAL } numberStil=UNKNOWN;
    
  if (  ((unsigned char) toupper(*tokenSign)) == (unsigned char)'X' )
  {
    numberStil=HEX;
    tokenSign++;
  }
  else if (((unsigned char) toupper(*tokenSign)) == (unsigned char)'O' )
  {
    numberStil=OCTAL;
    tokenSign++;      
  }
  else if ( isdigit(*tokenSign) )
  {
    numberStil=DECIMAL;
  }
    
  if (numberStil==UNKNOWN)
    return false;

  string utf8DecodedToken="";
  while(    (tokenSign !=pUtf8Token.end())
      	&&  ( 	((unsigned char)*tokenSign) 
	    !=  ((unsigned char)';')
	    ) 
      	) 
  {    
    utf8DecodedToken+=*tokenSign;
    tokenSign++;
  }
  
  unsigned int signValue=0;
  switch(numberStil)
  {
    case HEX:
      sscanf(utf8DecodedToken.c_str(),"%x",&signValue);
        break;
    case OCTAL:
      sscanf(utf8DecodedToken.c_str(),"%o",&signValue);
        break;
    default:
      sscanf(utf8DecodedToken.c_str(),"%u",&signValue);
  }
  pSign=(unsigned char) signValue;
  return true;  
}    

/** 
 *  \brief find next still not decoded (xml/html) token
 *
 *  implementation is valid for xml- and html-tokens
 *  \param pUnDecodedToken :- string which shall be filled
 *  \return true if pUnDecodedToken has been filled otherwise false
 */  
bool krmuo::Converter::getNextUnDecodedToken(string& pUnDecodedToken)
{
  // are there still candidates for decoding?
  if (mUnConvertedStringIterator==mUnConvertedStringEnd)
    return false;
    
  pUnDecodedToken="";
  
  // take the start position of the next candidate
  string::const_iterator
    unDecodedParseIterator=mUnConvertedStringIterator;
  
  // is the candidate more than a simple token
  if (  ( (unsigned char)*unDecodedParseIterator )
     == ( (unsigned char) '&')
     )
  {
    bool doParse=true;  ///< note that we still have to parse
    
    while(doParse)
    {
      // did we run into uncorrect html
      if (  (unDecodedParseIterator==mUnConvertedStringEnd)
         ||
            (   (unsigned char)*unDecodedParseIterator 
            <=  (unsigned char)' '
          ) )
      {
        
        // ... then eval the first sign (=&) as normal sign
        unDecodedParseIterator=mUnConvertedStringIterator;
        pUnDecodedToken="";
        
        // each last sign of an html-tag will 
        // be overtaken at the end of this method 
        // and in this case the last sign  is the first sign
        doParse=false;
      }
      else
      {
        // have we reached the correct end of an html-tag?
        if  (   (unsigned char)*unDecodedParseIterator 
            ==  (unsigned char)';'
            )
        {
          // each last sign of an html-sign-tag will 
          // be overtaken at the end of this method 
          doParse=false;
        }
        else
        {
          // overtake the signtagsign into the token-string
          // and test the next signtagsign
          pUnDecodedToken+=*unDecodedParseIterator;
          unDecodedParseIterator++;
        }        
      }
    }    
    mUnConvertedStringIterator=unDecodedParseIterator;
  }
  
  // take over the last still not overtaken sign of the token:
  // if the token is only a simple the sign this (first) sign
  // is also the last sign
  pUnDecodedToken+=*mUnConvertedStringIterator;
  mUnConvertedStringIterator++;
  return true;
}

/**
 *  \brief utf8-encodable signs have values >= 0x80
 *  \param pSign :- sign which shall be tested
 *  \param pUtf8Token :- contains the encodation result if encodation is possible
 *  \return true if val(pSign)>=0x80 otherwise false
 */
bool krmuo::Converter::isUtf8EncodableSign(const unsigned char pSign, string& pUtf8Token)
{
  if (pSign < 0x80)
    return false;
  sprintf(mUtf8Sign,"&#%02d;", pSign);
  pUtf8Token=mUtf8Sign;
  return true;
}

/* &&& (4) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&&     methods for class HtmlConverter &&&&&&&&&&&&&&&&&&&& */
/* --- (4.A) - public ----------------------------------------- */
/* :-: (4.A.1) public constructors, inits & operators :-:-:-:-: */
/** 
 * \brief constructor for an HtmlConverter
 * the most important task is to initialize the generally used 
 * pointer to a CodetokenMap with the address of the
 * HtmlConverter specific HtmlCodeTokenMap
 */
krmuo::HtmlConverter::HtmlConverter()
{
  mTokenMap=&mHtmlCodeTokenMap; 
}

/** 
 * \brief deconstructor for an HtmlConverter
 */
krmuo::HtmlConverter::~HtmlConverter()
{
}

/* &&& (5) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&&     methods for class XmlConverter &&&&&&&&&&&&&&&&&&&& */
/* --- (5.A) - public ----------------------------------------- */
/* :-: (5.A.1) public constructors, inits & operators :-:-:-:-: */
/** 
 * \brief constructor for an HtmlConverter
 * the most important task is to initialize the generally used 
 * pointer to a CodetokenMap with the address of the
 * XmlConverter specific XmlCodeTokenMap
 */
krmuo::XmlConverter::XmlConverter()
{
  mTokenMap=&mXmlCodeTokenMap; 
}

/** 
 * \brief deconstructor for an XmlConverter
 */
krmuo::XmlConverter::~XmlConverter()
{
}

/* &&& (6) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&&     methods for class UrlConverter &&&&&&&&&&&&&&&&&&&& */
/* --- (6.A) - public ----------------------------------------- */
/* :-: (6.A.1) public constructors, inits & operators :-:-:-:-: */
/** 
 * \brief constructor for an UrlConverter
 * initializes pointer to a CodetokenMap with the address of the
 * UrllConverter specific UrlCodeTokenMap.
 *
 * Note: this address isn't used because all the general
 * methods of the class Converter are overloaded by the
 * UrlConverter specific methods
 */
krmuo::UrlConverter::UrlConverter()
{
  mTokenMap=&mUrlCodeTokenMap; 
}

/** 
 * \brief deconstructor for an UrlConverter
 */
krmuo::UrlConverter::~UrlConverter()
{
}
/* :-: (6.A.4) public others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/** 
 *  \brief realize a decodation valid for url-encoded strings
 *
 *  \param pUnDecodedString :- a row of undecoded entities
 *  \return a copy of the class internal result string
 *
 *  while any undecoded entity can be found
 *  decoded that entity and append the result
 *  ( normally a one-sign-string )
 *  to the internal result string of the conversion
 */
const string krmuo::UrlConverter::decode(const string& pUnDecodedString)
{

  mUnConvertedStringEnd=pUnDecodedString.end();
  mUnConvertedStringIterator=pUnDecodedString.begin();
  mConvertedString="";
  unsigned char sign;
  while(getNextUnDecodedToken(mUnConvertedToken))
  {
    
    if (isDecodableEmbeddedUtf8Token(mUnConvertedToken,sign))
      mConvertedString+=sign;
    else if (mUrlCodeTokenMap.isDecodableToken(mUnConvertedToken))
      mConvertedString+=mUrlCodeTokenMap.decodeToken(mUnConvertedToken);
    else
      mConvertedString+=mUnConvertedToken;
  }  
  return mConvertedString;
}

/** 
 *  \brief realize an encodation
 *
 *  \param pUnEncodedString :- a row of unencoded signs
 *  \return a copy of the class internal result string
 *
 *  while any undecoded signs can be found
 *  encoded that sign and append the result (= entity(!))
 *  to the internal result string of the conversion
 */
const string krmuo::UrlConverter::encode(const string& pUnEncodedString)
{
  mUnConvertedStringIterator=pUnEncodedString.begin();
  mConvertedString="";
  while(mUnConvertedStringIterator!=pUnEncodedString.end())
  {
    if (mUrlCodeTokenMap.isEncodableSign((unsigned char)*mUnConvertedStringIterator))
      mConvertedString+=mUrlCodeTokenMap.encodeSign((unsigned char)*mUnConvertedStringIterator);
    else if (isUtf8EncodableSign((unsigned char)*mUnConvertedStringIterator,mUtf8String))
      mConvertedString+=mUtf8String;
    else
      mConvertedString+=(unsigned char)*mUnConvertedStringIterator;
    mUnConvertedStringIterator++;
  }
  return mConvertedString;
}

/** 
 *  \brief find next still not decoded url token
 *
 *  special decoded signs have the structure %xx where
 *  xx denote a hexcode of the sign 
 *  IS THAT REALLY RIGHT???
 *
 *  implementation is valid for url-tokens
 *  \param pUnDecodedToken :- string which shall be filled
 *  \return true if pUnDecodedToken has been filled otherwise false
 */  
bool krmuo::UrlConverter::getNextUnDecodedToken(string& pUnDecodedToken)
{
  // are there still candidates for decoding?
  if (mUnConvertedStringIterator==mUnConvertedStringEnd)
    return false;
    
  pUnDecodedToken="";
  
  // take the start position of the next candidate
  string::const_iterator
    unDecodedParseIterator=mUnConvertedStringIterator;
  
  // is the candidate more than a simple token
  if (  ( (unsigned char)*unDecodedParseIterator )
     == ( (unsigned char) '%')
     )
  {
    unDecodedParseIterator++;
    if (unDecodedParseIterator==mUnConvertedStringEnd)
      unDecodedParseIterator=mUnConvertedStringIterator;
    else
    {
      pUnDecodedToken="%";
      pUnDecodedToken+=*unDecodedParseIterator;
      unDecodedParseIterator++;
      if (unDecodedParseIterator==mUnConvertedStringEnd)
      	unDecodedParseIterator=mUnConvertedStringIterator;
    }
  }
  mUnConvertedStringIterator=unDecodedParseIterator;
  
  // take over the last still not overtaken sign of the token:
  // if the token is only a simple the sign this (first) sign
  // is also the last sign
  pUnDecodedToken+=*mUnConvertedStringIterator;
  mUnConvertedStringIterator++;
  return true;
}

/**
 *  \brief utf8-encodable signs have values >= 0x80
 *  \param pSign :- sign which shall be tested
 *  \param pUtf8Token :- contains the encodation result if encodation is possible
 *  \return true if val(pSign)>=0x80 otherwise false
 */
bool krmuo::UrlConverter::isUtf8EncodableSign(const unsigned char pSign, string& pUtf8Token)
{
  if (pSign < 0x80)
    return false;
  sprintf(mUtf8Sign,"%c%02x",'%',(unsigned int)pSign);     
  pUtf8Token=mUtf8Sign;
  return true;
}

/**
 *  \brief utf8-decodable tokens have the structure %..
 *  \param pUtf8Token :- token which shall be tested
 *  \param pSign :- reference to unsigned char into which the value will be stored
 *  \return true if token is utf8decodable and content of pSign is valid
 */
bool krmuo::UrlConverter::isDecodableEmbeddedUtf8Token
( const string& pUtf8Token,
  unsigned char& pSign
)
{
  if (pUtf8Token.size()<3)
    return false;
    
  string::const_iterator tokenSign=pUtf8Token.begin(); 
  if (((unsigned char)* tokenSign)  != ((unsigned char)'%'))
    return false;

  tokenSign++;

  string utf8DecodedToken="";
  while(tokenSign !=pUtf8Token.end())
  {    
    utf8DecodedToken+=*tokenSign;
    tokenSign++;
  }
  
  unsigned int signValue=0;
  sscanf(utf8DecodedToken.c_str(),"%x",&signValue);
  pSign=(unsigned char) signValue;
  return true;  
}    
