/*
 * Copyright (C) 2001 Karsten Reincke <karsten.reincke@gnukose.org>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 * file <muo_convert.h> version <#1.3.1#> of project <MrProjext>
 */   

/** \file muo_convert.h
 *  \brief includable declarations of and for the conversion of strings
 *
 *  \author karsten reincke
 */

#ifndef  CONVERTER_H
#define  CONVERTER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <map>
#include "muo_convert_table.h"
/* &&& exported preprocessor-defines &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */

/* &&& class-declarations &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */

using namespace std;

namespace krmuo
{

/**
 *  \brief object for converting strings
 *   
 *  Converter are XmlConverter, UrlConverter, HtmlConverter
 *
 *  Note: Html- and xml-encodation uses the same tag-structure
 *        namely &...; Therefore the only difference between
 *        an thml- and and xml-converter concernes the set
 *        of tokens being fixed in the codeTokenMap.
 *        =>
 *        The default-implementation of a converter is valid
 *        vor an xml-converter and an html-converter. The
 *        difference is realized by setting the pointer
 *        mTokenMap to the correct codeTokenMap - the
 *        htmlCodeTokenMap or the xmlCodeTokenMap - which
 *        are made as member by the constructors of the
 *    	  real converters
 */

class Converter
{
  public:
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  protected:    
    string::const_iterator 
      mUnConvertedStringIterator;   ///< start for the next get-token-method
    string::const_iterator 
      mUnConvertedStringEnd;  	    ///< start for the next get-token-method
    string mUnConvertedToken;       ///< the last unconverted token
    string mConvertedToken;         ///< the last converted token
    string mConvertedString;  	    ///< the result of the conversion
    char mUtf8Sign[20];       	    ///< buffer for generating an utfString
    string mUtf8String;       	    ///< ... and the corresponding string itself
    
    krmuo::CodeTokenMap* mTokenMap; ///< pointer the valid codeTokenMap  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
    /** 
     * \brief general constructor for any type of Converter
     */
    Converter();
    virtual ~Converter();                 ///< class-destructor
           
    /** 
     *  \brief realize a (default) decodation (valid for xml and html)
     */
    const string decode(const string& pUnDecodedString);
    
    /** 
     *  \brief realize a (default) encodation (valid for xml and html)
     */
    const string encode(const string& pUnEncodedString);

  private:    
    /**
     *	utf8-decodable (xml/html) tokens have the structure &#...;
     */
    bool isDecodableEmbeddedUtf8Token(const string& pUtf8Token, unsigned char& pSign);    

    /** 
     *  \brief find next still not decoded (xml/html) token
     */  
    bool getNextUnDecodedToken(string& pUnDecodedToken);
 
    /**
     *	\brief utf8-encodable signs have values >= 0x80
     */
    bool isUtf8EncodableSign(const unsigned char pSign, string& pUtf8Token);
     
};

/**
 *  \brief object for html-encode or html-decode strings
 */

class HtmlConverter:public Converter
{

  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  private:
    krmuo::HtmlCodeTokenMap mHtmlCodeTokenMap;  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /** 
     * \brief constructor for an HtmlConverter
     */
    HtmlConverter();
    virtual ~HtmlConverter();                 ///< class-destructor
        
};

/**
 *  \brief object for xml-encode or xml-decode strings
 */

class XmlConverter:public Converter
{

  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  private:
    krmuo::XmlCodeTokenMap mXmlCodeTokenMap;  
    
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /** 
     * \brief constructor for an XmlConverter
     */
    XmlConverter();
    virtual ~XmlConverter();                 ///< class-destructor

};

/**
 *  \brief object for url-encode or url-decode strings
 */

class UrlConverter:public Converter
{

  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  private:
    krmuo::UrlCodeTokenMap mUrlCodeTokenMap;  
    
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /** 
     * \brief constructor for an UrlConverter
     */
    UrlConverter();
    virtual ~UrlConverter();                 ///< class-destructor
    /** 
     *  \brief realize a decodation valid for url-encoded strings
     */
    const string decode(const string& pUnDecodedString);
    
    /** 
     *  \brief realize an encodation valid for url-encoded strings
     */
    const string encode(const string& pUnEncodedString);
         
  private:
    /**
     *	\brief utf8-decodable tokens have the structure &#...;
     */
    bool isDecodableEmbeddedUtf8Token(const string& pUtf8Token, unsigned char& pSign);    

    /** 
     *  \brief find next still not decoded token
     */  
    bool getNextUnDecodedToken(string& pUnDecodedToken);
 
    /**
     *	\brief utf8-encodable signs have values >= 0x80
     */
    bool isUtf8EncodableSign(const unsigned char pSign, string& pUtf8Token);
};


}

#endif
