/*
 * Copyright (C) 2001 Karsten Reincke <karsten.reincke@gnukose.org>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 * file <muo_convert_table.h> version <#1.3.1#> of project <MrProjext>
 */   

/** \file muo_convert_table.h
 *  \brief includable declarations of and for the conversion of strings
 *
 *  \author karsten reincke
 */

#ifndef  CODETOKENMAP_H
#define  CODETOKENMAP_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <map>
/* &&& exported preprocessor-defines &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */

/* &&& class-declarations &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */

using namespace std;

namespace krmuo
{

/**
 *  \brief for managing the content of tokens for codations
 *
 *  each type of encodation-decodation will own a specific
 *  pair of sign-token-maps by which signs are linked to
 *  tokens and vc. these maps need be filled only once.
 *  and the use of these maps can the same for each type
 *  of encodation. this is realized using the following method:
 *
 *  each class being derivated from class CodeTokenMap owns
 *  specific static maps for linking signs to tokens and vc.
 *  these static maps will be filled by the constructor of
 *  these derivated subclasses. each of these subconstructors
 *  stores the addresses of theses maps into the (not stictic)
 *  pointervariables being offered by their motherclass
 *  CodeTokenMap.
 *
 *  and this motherclass offers the common methods for
 *  transposing a sign into a token and vc.
 */

class CodeTokenMap
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  protected:
  
    const string* mClearToken;	      	    ///< pointer to the empty token
    map<string,unsigned char>* mDecoderMap; ///< map key = encoded / value = clear
    map<unsigned char,string>* mEncoderMap; ///< map key = clear / value = encoded

    
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class CodeTokenMap
     */
    CodeTokenMap();           
        
    virtual ~CodeTokenMap();                      ///< class-destructor
        
  
  public:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /**
     *  \brief is the given sign encodable?
     */
    virtual bool isEncodableSign(unsigned char pSign) const;
    
    /**
     *  \brief encode the given sign
    */
    virtual const string& encodeSign(unsigned char pSign) const;
    

    /**
     *  \brief is the given token decodable ?
     */

    bool isDecodableToken(const string& pToken) const;

    /**
     *  \brief returns sign for which the given entity stands
     */
    virtual unsigned char decodeToken(const string& pToken) const;

    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /** \brief writes encodation tabel to stdout */
    inline void dumpEncodeTable()
    {
      map<unsigned char,string>::const_iterator i=mEncoderMap->begin();
      while(i!=mEncoderMap->end())
      {
        cout << "[" << i->first << "][" << i->second << "]\n";
      	i++;
      }
    }

    /** \brief writes decodation tabel to stdout */
    inline void dumpDecodeTable()
    {
      map<string,unsigned char>::const_iterator i=mDecoderMap->begin();
      while(i!=mDecoderMap->end())
      {
        cout << "[" << i->first << "][" << i->second << "]\n";
      	i++;
      }
    }
    
    
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     CodeTokenMap(const CodeTokenMap& pCo);  ///< copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
};


/**
 *  \brief for managing the content of tokens for html-codation
 */

class HtmlCodeTokenMap:public CodeTokenMap
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    static const string mHtmlClearToken;
    static map<string,unsigned char> mHtmlDecoderMap; ///< map key = encoded / value = clear
    static map<unsigned char,string> mHtmlEncoderMap; ///< map key = clear / value = encoded
    
    static bool mHtmlMapsMustStillBeFilled; ///< does map already exist?
    
    static const unsigned char signValueAmp;   ///< clear sign ampercend
    static const string signEncodationAmp;          ///< html encoded ampercend
    static const unsigned char signValueLt;
    static const string signEncodationLt;
    static const unsigned char signValueGt;
    static const string signEncodationGt;
    static const unsigned char signValueApos;
    static const string signEncodationApos;
    static const unsigned char signValueQuot;
    static const string signEncodationQuot;
    
    static const unsigned char signValue160;
    static const string signEncodation160;
    static const unsigned char signValue161;
    static const string signEncodation161;
    static const unsigned char signValue162;
    static const string signEncodation162;
    static const unsigned char signValue163;
    static const string signEncodation163;
    static const unsigned char signValue164;
    static const string signEncodation164;
    static const unsigned char signValue165;
    static const string signEncodation165;
    static const unsigned char signValue166;
    static const string signEncodation166;
    static const unsigned char signValue167;
    static const string signEncodation167;
    static const unsigned char signValue168;
    static const string signEncodation168;
    static const unsigned char signValue169;
    static const string signEncodation169;
   
    static const unsigned char signValue170;
    static const string signEncodation170;
    static const unsigned char signValue171;
    static const string signEncodation171;
    static const unsigned char signValue172;
    static const string signEncodation172;
    static const unsigned char signValue173;
    static const string signEncodation173;
    static const unsigned char signValue174;
    static const string signEncodation174;
    static const unsigned char signValue175;
    static const string signEncodation175;
    static const unsigned char signValue176;
    static const string signEncodation176;
    static const unsigned char signValue177;
    static const string signEncodation177;
    static const unsigned char signValue178;
    static const string signEncodation178;
    static const unsigned char signValue179;
    static const string signEncodation179;

    static const unsigned char signValue180;
    static const string signEncodation180;
    static const unsigned char signValue181;
    static const string signEncodation181;
    static const unsigned char signValue182;
    static const string signEncodation182;
    static const unsigned char signValue183;
    static const string signEncodation183;
    static const unsigned char signValue184;
    static const string signEncodation184;
    static const unsigned char signValue185;
    static const string signEncodation185;
    static const unsigned char signValue186;
    static const string signEncodation186;
    static const unsigned char signValue187;
    static const string signEncodation187;
    static const unsigned char signValue188;
    static const string signEncodation188;
    static const unsigned char signValue189;
    static const string signEncodation189;

    static const unsigned char signValue190;
    static const string signEncodation190;
    static const unsigned char signValue191;
    static const string signEncodation191;
    static const unsigned char signValue192;
    static const string signEncodation192;
    static const unsigned char signValue193;
    static const string signEncodation193;
    static const unsigned char signValue194;
    static const string signEncodation194;
    static const unsigned char signValue195;
    static const string signEncodation195;
    static const unsigned char signValue196;
    static const string signEncodation196;
    static const unsigned char signValue197;
    static const string signEncodation197;
    static const unsigned char signValue198;
    static const string signEncodation198;
    static const unsigned char signValue199;
    static const string signEncodation199;

    static const unsigned char signValue200;
    static const string signEncodation200;
    static const unsigned char signValue201;
    static const string signEncodation201;
    static const unsigned char signValue202;
    static const string signEncodation202;
    static const unsigned char signValue203;
    static const string signEncodation203;
    static const unsigned char signValue204;
    static const string signEncodation204;
    static const unsigned char signValue205;
    static const string signEncodation205;
    static const unsigned char signValue206;
    static const string signEncodation206;
    static const unsigned char signValue207;
    static const string signEncodation207;
    static const unsigned char signValue208;
    static const string signEncodation208;
    static const unsigned char signValue209;
    static const string signEncodation209;

    static const unsigned char signValue210;
    static const string signEncodation210;
    static const unsigned char signValue211;
    static const string signEncodation211;
    static const unsigned char signValue212;
    static const string signEncodation212;
    static const unsigned char signValue213;
    static const string signEncodation213;
    static const unsigned char signValue214;
    static const string signEncodation214;
    static const unsigned char signValue215;
    static const string signEncodation215;
    static const unsigned char signValue216;
    static const string signEncodation216;
    static const unsigned char signValue217;
    static const string signEncodation217;
    static const unsigned char signValue218;
    static const string signEncodation218;
    static const unsigned char signValue219;
    static const string signEncodation219;

    static const unsigned char signValue220;
    static const string signEncodation220;
    static const unsigned char signValue221;
    static const string signEncodation221;
    static const unsigned char signValue222;
    static const string signEncodation222;
    static const unsigned char signValue223;
    static const string signEncodation223;
    static const unsigned char signValue224;
    static const string signEncodation224;
    static const unsigned char signValue225;
    static const string signEncodation225;
    static const unsigned char signValue226;
    static const string signEncodation226;
    static const unsigned char signValue227;
    static const string signEncodation227;
    static const unsigned char signValue228;
    static const string signEncodation228;
    static const unsigned char signValue229;
    static const string signEncodation229;

    static const unsigned char signValue230;
    static const string signEncodation230;
    static const unsigned char signValue231;
    static const string signEncodation231;
    static const unsigned char signValue232;
    static const string signEncodation232;
    static const unsigned char signValue233;
    static const string signEncodation233;
    static const unsigned char signValue234;
    static const string signEncodation234;
    static const unsigned char signValue235;
    static const string signEncodation235;
    static const unsigned char signValue236;
    static const string signEncodation236;
    static const unsigned char signValue237;
    static const string signEncodation237;
    static const unsigned char signValue238;
    static const string signEncodation238;
    static const unsigned char signValue239;
    static const string signEncodation239;

    static const unsigned char signValue240;
    static const string signEncodation240;
    static const unsigned char signValue241;
    static const string signEncodation241;
    static const unsigned char signValue242;
    static const string signEncodation242;
    static const unsigned char signValue243;
    static const string signEncodation243;
    static const unsigned char signValue244;
    static const string signEncodation244;
    static const unsigned char signValue245;
    static const string signEncodation245;
    static const unsigned char signValue246;
    static const string signEncodation246;
    static const unsigned char signValue247;
    static const string signEncodation247;
    static const unsigned char signValue248;
    static const string signEncodation248;
    static const unsigned char signValue249;
    static const string signEncodation249;

    static const unsigned char signValue250;
    static const string signEncodation250;
    static const unsigned char signValue251;
    static const string signEncodation251;
    static const unsigned char signValue252;
    static const string signEncodation252;
    static const unsigned char signValue253;
    static const string signEncodation253;
    static const unsigned char signValue254;
    static const string signEncodation254;
    static const unsigned char signValue255;
    static const string signEncodation255;

  protected:
    
 
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class CodeTokenMap
     */
    HtmlCodeTokenMap();           
        
    ~HtmlCodeTokenMap();                      ///< class-destructor
        
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     HtmlCodeTokenMap(const HtmlCodeTokenMap& pCo);  ///< copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
};

/**
 *  \brief for managing the content of tokens for xml-codation
 */
class XmlCodeTokenMap:public CodeTokenMap
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    static const string mXmlClearToken;
    static map<string,unsigned char> mXmlDecoderMap; ///< map key = encoded / value = clear
    static map<unsigned char,string> mXmlEncoderMap; ///< map key = clear / value = encoded
    
    static bool mXmlMapsMustStillBeFilled; ///< does map already exist?
    
    static const unsigned char signValueAmp;
    static const string signEncodationAmp;
    static const unsigned char signValueLt;
    static const string signEncodationLt;
    static const unsigned char signValueGt;
    static const string signEncodationGt;
    static const unsigned char signValueApos;
    static const string signEncodationApos;
    static const unsigned char signValueQuot;
    static const string signEncodationQuot;
   
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* &&& constructors / init / operators &&&&&&&&&&&&&&&&&&&&&&&&&&& */
    
    /** 
      * \brief constructor for class XmlCodeTokenMap
      */
    XmlCodeTokenMap();                       
    
    ~XmlCodeTokenMap();                      ///< class-destructor
        
   
  private:
    XmlCodeTokenMap(const XmlCodeTokenMap& co); ///< copy-constructor
      
  protected:
  
};

/**
 *  \brief for managing the content of tokens for url-codation
 */
class UrlCodeTokenMap:public CodeTokenMap
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    static const string mUrlClearToken;
    static map<string,unsigned char> mUrlDecoderMap; ///< map key = encoded / value = clear
    static map<unsigned char,string> mUrlEncoderMap; ///< map key = clear / value = encoded
    
    static bool mUrlMapsMustStillBeFilled; ///< does map already exist?

    static const unsigned char signValueSemicolon;
    static const string signEncodationSemicolon;
    static const unsigned char signValueSlash;
    static const string signEncodationSlash;
    static const unsigned char signValueQuest;
    static const string signEncodationQuest;
    static const unsigned char signValueColon;
    static const string signEncodationColon;
    static const unsigned char signValueAtsign;
    static const string signEncodationAtsign;
    static const unsigned char signValueEqual;
    static const string signEncodationEqual;
    static const unsigned char signValueAmpercent;
    static const string signEncodationAmpercent;
    static const unsigned char signValueSpace;
    static const string signEncodationSpace;
    static const unsigned char signValueLt;
    static const string signEncodationLt;
    static const unsigned char signValueGt;
    static const string signEncodationGt;
    static const unsigned char signValueQuote;
    static const string signEncodationQuote;
    static const unsigned char signValueGadder;
    static const string signEncodationGadder;
    static const unsigned char signValuePercent;
    static const string signEncodationPercent;
    static const unsigned char signValueLsBracket;
    static const string signEncodationLsBracket;
    static const unsigned char signValueRsBracket;
    static const string signEncodationRsBracket;
    static const unsigned char signValueLBrace;
    static const string signEncodationLBrace;
    static const unsigned char signValueRBrace;
    static const string signEncodationRBrace;
    static const unsigned char signValueVBar;
    static const string signEncodationVBar;
    static const unsigned char signValueBackSlash;
    static const string signEncodationBackSlash;
    static const unsigned char signValueCaret;
    static const string signEncodationCaret;
    static const unsigned char signValueTilde;
    static const string signEncodationTilde;
    static const unsigned char signValueBackQuote;
    static const string signEncodationBackQuote;
   
    static const unsigned char signValuePlusSign;
    static const string signEncodationPlusSign;
    static const unsigned char signValueBlank;
    static const string signEncodationBlank;
  public:
    /* &&& constructors / init / operators &&&&&&&&&&&&&&&&&&&&&&&&&&& */
    
    /** 
      * \brief constructor for class XmlCodeTokenMap
      */
    UrlCodeTokenMap();                       
    
    ~UrlCodeTokenMap();                      ///< class-destructor
        
   
  private:
    UrlCodeTokenMap(const UrlCodeTokenMap& co); ///< copy-constructor

};


}

#endif
