/*
 * Copyright (C) 2001 Karsten Reincke <reincke@gnukose.org>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 * file <muo_logging.cc> version <#1.3.1#> of project <MrProjext>
 */   

/** \file muo_logging.cc
 *  \brief all definitions of and for classes for logging methods
 *
 *  \author karsten reincke
 */

#include <iostream>
#include <time.h>
#include "muo_logging.h"
using namespace krmuo;
using namespace std;
/* &&& (1) local preprocessor-defines &&&&&&&&&&&&&&&&&&&&&&& */

/* &&& (2) definitions of static global variables &&&&&&&&&&& */
          // note: they mustnt again be declared as static
 
/* &&& (3) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&&     methods for class LoggingSuitcase &&& */

/* --- (3.A) public ------------------------------------------ */

/* :-: (3.A.1) public constructors, inits & operators -:-:-:-: */

/**
 *  \brief constructor for LoggingUnits without fileentries
 *  
 *  Constructor for LoggingUnits beeing able to write messages onto the screen
 *  \param pMaxVisibleMessageLevel :- {SHOW_NM(=0),SHOW_SM(=1),SHOW_SMADM(=2),SHOW_SMADMAHM(=3)}
 *  \param pScreen :- { stdout, stderr }
 */
LoggingUnit::LoggingUnit
( const unsigned int pMaxVisibleMessageLevel,
  FILE *pScreen)
: mMaxVisibleMessageLevel(pMaxVisibleMessageLevel),
  mScreen(pScreen),
  mLogFileName(""),
  mFile(NULL),
  mUsesSharedLoggingFile(false)
{
}

/** 
 *  \brief constructor for LoggingUnits without screen-entries
 *
 *  Logging Unit beeing able to write messages into a logfile
 *  \param pMaxVisibleMessageLevel :- {SHOW_NM(=0),SHOW_SM(=1),SHOW_SMADM(=2),SHOW_SMADMAHM(=3)}
 *  \param pLogFileName = name of the logfile
 *  \param pUsesSharedLoggingFile :- { USE_SHARED_LOGFILE, USE_OWN_LOGFILE }
 */
LoggingUnit::LoggingUnit
( const unsigned int pMaxVisibleMessageLevel,
  const string& pLogFileName,
  bool pUsesSharedLoggingFile)
: mMaxVisibleMessageLevel(pMaxVisibleMessageLevel),
  mScreen(NULL),
  mLogFileName(pLogFileName),
  mFile(NULL),
  mUsesSharedLoggingFile(pUsesSharedLoggingFile)
{
  if (!(mUsesSharedLoggingFile))
  {
    mFile=fopen(mLogFileName.c_str(),"a");
    if (!(mFile))
    {
      mScreen=stderr;
      cerr << "Can't open logfile <" << pLogFileName<<">\n";
    }
  }
}

/**
 *  \brief constructor for LoggingUnits with screen- and fileentries
 *  
 *  LoggingUnit beeing able to write messages onto the screen and into a logfile
 *  \param pMaxVisibleMessageLevel :- {SHOW_NM(=0),SHOW_SM(=1),SHOW_SMADM(=2),SHOW_SMADMAHM(=3)}
 *  \param pScreen :- { stdout, stderr }
 *  \param pLogFileName = name of the logfile
 *  \param pUsesSharedLoggingFile :- { USE_SHARED_LOGFILE, USE_OWN_LOGFILE }
 */
LoggingUnit::LoggingUnit
( const unsigned int pMaxVisibleMessageLevel,
  FILE *pScreen,
  const string& pLogFileName,
  bool pUsesSharedLoggingFile)
: mMaxVisibleMessageLevel(pMaxVisibleMessageLevel),
  mScreen(pScreen),
  mLogFileName(pLogFileName),
  mFile(NULL),
  mUsesSharedLoggingFile(pUsesSharedLoggingFile)
{
  if (!(mUsesSharedLoggingFile))
  {
    mFile=fopen(mLogFileName.c_str(),"a");
    if (!(mFile))
    {
      mScreen=stderr;
      cerr << "Can't open logfile <" << pLogFileName<<">\n";
    }
  }
}

/**
 * \brief class-destructor
 *
 *  class-destructor which closes the logginfiles if nescessary
 */
LoggingUnit::~LoggingUnit()
{
  if (mFile)
    fclose(mFile);
}

/* :-: (3.A.2) public getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (3.A.3) public setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (3.A.4) public others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/**
 * \brief write one2one-string-message into the LoggingUnit
 *
 * write one-string-message into the LoggingUnit 
 * (= into file and/or onto screen)
 * \param pLogString :- the message
 * \param pLogEntryLevel :- { SMESS,DMESS, HMESS }
 */
void LoggingUnit::makeEntry
( const string& pLogString,
  const unsigned int pLogEntryLevel
) const
{
  if (pLogEntryLevel<=mMaxVisibleMessageLevel)
  {
    string timeLabel="";
    fillTimeLabel(timeLabel);  
    if (mScreen)
    {
      fprintf(mScreen,"%s (LL%d): %s\n",timeLabel.c_str(),pLogEntryLevel,pLogString.c_str());
    }
    FILE *lFile;
    if (mUsesSharedLoggingFile)
      lFile=fopen(mLogFileName.c_str(),"a");
    else
      lFile=mFile;
    if (lFile)
    {
      fprintf(lFile,"%s (LL%d): %s\n",timeLabel.c_str(),pLogEntryLevel,pLogString.c_str());
      fflush(lFile);
      if (mUsesSharedLoggingFile)
      {
      	fclose(lFile);
      }
    }    
  }
}
/**
 * \brief write two2one-string-message into the LoggingUnit
 *
 * write string-string-message as one-string-message into the LoggingUnit 
 * (= into file and/or onto screen)
 * \param pLogString1 :- first part of the message
 * \param pLogString2 :- second part of the message
 * \param pLogEntryLevel :- { SMESS,DMESS, HMESS }
 */
void LoggingUnit::makeEntry
( const string& pLogString1,
  const string& pLogString2, 
  const unsigned int pLogEntryLevel
) const
{
  if (pLogEntryLevel<=mMaxVisibleMessageLevel)
  {    
    string timeLabel="";
    fillTimeLabel(timeLabel);  
    if (mScreen)
    {
      fprintf(mScreen,"%s (LL%d): %s: <%s>\n",timeLabel.c_str(),pLogEntryLevel,pLogString1.c_str(),pLogString2.c_str());
    }
    FILE *lFile;  
    if (mUsesSharedLoggingFile)
      lFile=fopen(mLogFileName.c_str(),"a");
    else
      lFile=mFile;
    if (lFile)
    {
      fprintf(lFile,"%s (LL%d): %s: <%s>\n",timeLabel.c_str(),pLogEntryLevel,pLogString1.c_str(),pLogString2.c_str());
      fflush(lFile);
      if (mUsesSharedLoggingFile)
      {
      	fclose(lFile);
      }
    }    
  }  
}

/**
 * \brief write three2one-string-message into the LoggingUnit
 *
 * write string-string-message as one-string-message into the LoggingUnit 
 * (= into file and/or onto screen)
 * \param pLogString1 :- first part of the message
 * \param pLogString2 :- second-a part of the message
 * \param pLogString3 :- second-b part of the message
 * \param pLogEntryLevel :- { SMESS,DMESS, HMESS }
 */
void LoggingUnit::makeEntry
( const string& pLogString1,
  const string& pLogString2, 
  const string& pLogString3,  
  const unsigned int pLogEntryLevel
)const
{
  if (pLogEntryLevel<=mMaxVisibleMessageLevel)
  {    
    string timeLabel="";
    fillTimeLabel(timeLabel);  
    if (mScreen)
    {
      fprintf(mScreen,"%s (LL%d): %s: <%s [%s]>\n",
      	timeLabel.c_str(),pLogEntryLevel,
	  pLogString1.c_str(),pLogString2.c_str(),pLogString3.c_str());
    }
    FILE *lFile;  
    if (mUsesSharedLoggingFile)
      lFile=fopen(mLogFileName.c_str(),"a");
    else
      lFile=mFile;
    if (lFile)
    {
      fprintf(lFile,"%s (LL%d): %s: <%s [%s]>\n",
      	timeLabel.c_str(),pLogEntryLevel,
	  pLogString1.c_str(),pLogString2.c_str(),pLogString3.c_str());
      fflush(lFile);
      if (mUsesSharedLoggingFile)
      {
      	fclose(lFile);
      }
    }    
  }  
}

/**
 * \brief write two2oneOrTwo-string-message into the LoggingUnit
 *
 * write string-string-message as one- or two-string message into the LoggingUnit 
 * (= into file and/or onto screen)
 * \param pLogString1 :- first part of the message
 * \param pLogString2 :- second part of the message
 * \param pLogEntryLevel :- { SMESS,DMESS, HMESS }
 * \param pTwoLine :- { USE_TWOLINES, USE_ONELINE }
 */
void LoggingUnit::makeEntry
( const string& pLogString1,
  const string& pLogString2, 
  const unsigned int pLogEntryLevel,
  bool pTwoLine
) const
{
  if (pLogEntryLevel<=mMaxVisibleMessageLevel)
  {    
    string timeLabel="";
    fillTimeLabel(timeLabel);  
    if (mScreen)
    {
      if (pTwoLine)
      	fprintf(mScreen,"%s (LL%d): %s:\n%s\n",timeLabel.c_str(),pLogEntryLevel,pLogString1.c_str(),pLogString2.c_str());
      else
      	fprintf(mScreen,"%s (LL%d): %s: <%s>\n",timeLabel.c_str(),pLogEntryLevel,pLogString1.c_str(),pLogString2.c_str());
    }
    FILE *lFile;  
    if (mUsesSharedLoggingFile)
      lFile=fopen(mLogFileName.c_str(),"a");
    else
      lFile=mFile;
    if (lFile)
    {
      if (pTwoLine)
      	fprintf(lFile,"%s (LL%d): %s:\n%s\n",timeLabel.c_str(),pLogEntryLevel,pLogString1.c_str(),pLogString2.c_str());
      else
      	fprintf(lFile,"%s (LL%d): %s: <%s>\n",timeLabel.c_str(),pLogEntryLevel,pLogString1.c_str(),pLogString2.c_str());
      fflush(lFile);
      if (mUsesSharedLoggingFile)
      {
      	fclose(lFile);
      }
    }       
  }
}

/**
 * \brief write three2oneOrTwo-string-message into the LoggingUnit
 *
 * write string-string-message as one- or two-string message into the LoggingUnit 
 * (= into file and/or onto screen)
 * \param pLogString1 :- first part of the message
 * \param pLogString2 :- second-a part of the message
 * \param pLogString3 :- second-b part of the message
 * \param pLogEntryLevel :- { SMESS,DMESS, HMESS }
 * \param pTwoLine :- { USE_TWOLINES, USE_ONELINE }
 */
void LoggingUnit::makeEntry
( const string& pLogString1,
  const string& pLogString2, 
  const string& pLogString3, 
  const unsigned int pLogEntryLevel,
  bool pTwoLine
) const
{
  if (pLogEntryLevel<=mMaxVisibleMessageLevel)
  {    
    string timeLabel="";
    fillTimeLabel(timeLabel);  
    if (mScreen)
    {
      if (pTwoLine)
      	fprintf(mScreen,"%s (LL%d): %s:\n%s [%s]\n",
	  timeLabel.c_str(),pLogEntryLevel,
	    pLogString1.c_str(),pLogString2.c_str(),pLogString3.c_str());
      else
      	fprintf(mScreen,"%s (LL%d): %s: <%s [%s]>\n",
	  timeLabel.c_str(),pLogEntryLevel,
	    pLogString1.c_str(),pLogString2.c_str(),pLogString3.c_str());
    }
    FILE* lFile;
    if (mUsesSharedLoggingFile)
      lFile=fopen(mLogFileName.c_str(),"a");
    else
      lFile=mFile;
    if (lFile)
    {
      if (pTwoLine)
      	fprintf(lFile,"%s (LL%d): %s:\n%s [%s]\n",
	  timeLabel.c_str(),pLogEntryLevel,
	    pLogString1.c_str(),pLogString2.c_str(),pLogString3.c_str());
      else
      	fprintf(lFile,"%s (LL%d): %s: <%s [%s]>\n",
	  timeLabel.c_str(),pLogEntryLevel,
	    pLogString1.c_str(),pLogString2.c_str(),pLogString3.c_str());
      fflush(lFile);
      if (mUsesSharedLoggingFile)
      {
      	fclose(lFile);
      }
    }       
  }
}

/**
 * \brief write string-uint-2-one-string-message into the LoggingUnit
 *
 * write string-uint-message as one-string-message into the LoggingUnit 
 * (= into file and/or onto screen)
 * \param pLogString :- string part of the message
 * \param pLogUint :- unsigned value of the message
 * \param pLogEntryLevel :- { SMESS,DMESS, HMESS }
 */
void LoggingUnit::makeEntry
( const string& pLogString,
  const unsigned int pLogUint, 
  const unsigned int pLogEntryLevel
)const
{
  if (pLogEntryLevel<=mMaxVisibleMessageLevel)
  {    
    string timeLabel="";
    fillTimeLabel(timeLabel);  
    if (mScreen)
    {
      fprintf(mScreen,"%s (LL%d): %s <%d>\n",timeLabel.c_str(),pLogEntryLevel,pLogString.c_str(),pLogUint);
    }
    FILE* lFile;
    if (mUsesSharedLoggingFile)
      lFile=fopen(mLogFileName.c_str(),"a");
    else
      lFile=mFile;
    if (lFile)
    {
      fprintf(lFile,"%s (LL%d): %s <%d>\n",timeLabel.c_str(),pLogEntryLevel,pLogString.c_str(),pLogUint);
      fflush(lFile);
      if (mUsesSharedLoggingFile)
      {
      	fclose(lFile);
      }
    }    
  }  
}

/**
 * \brief write string-int-2-one-string-message into the LoggingUnit
 *
 * write string-int-message as one-string-message into the LoggingUnit 
 * (= into file and/or onto screen)
 * \param pLogString :- string part of the message
 * \param pLogInt :- signed value for the message
 * \param pLogEntryLevel :- { SMESS,DMESS, HMESS }
 */
void LoggingUnit::makeEntry
( const string& pLogString,
  const int pLogInt, 
  const unsigned int pLogEntryLevel
)const
{
  if (pLogEntryLevel<=mMaxVisibleMessageLevel)
  {
    string timeLabel="";
    fillTimeLabel(timeLabel);  
    if (mScreen)
    {
      fprintf(mScreen,"%s (LL%d): %s <%d>\n",timeLabel.c_str(),pLogEntryLevel,pLogString.c_str(),pLogInt);
    }
    FILE* lFile;    
    if (mUsesSharedLoggingFile)
      lFile=fopen(mLogFileName.c_str(),"a");
    else
      lFile=mFile;
    if (lFile)
    {
      fprintf(lFile,"%s (LL%d): %s <%d>\n",timeLabel.c_str(),pLogEntryLevel,pLogString.c_str(),pLogInt);
      fflush(lFile);
      if (mUsesSharedLoggingFile)
      {
      	fclose(lFile);
      }
    }    
  }  
}

/* --- (3.B) - private ---------------------------------------- */
/* :-: (3.B.1) private constructors, inits & operators -:-:-:-: */

/**
 *  \brief copy-constructor
 *
 *  class copy-constructor (mustn't be called)
 *
 *  \param co constant reference onto copyable object
 *  \return new object with duplicated content
 *
LoggingUnit::LoggingUnit(const LoggingUnit& co)
{ 
  assert(0);
}
*/

/* :-: (3.B.2) private getter -:-:-:-:-:-:-:-:-:-::-:-:-:-:-:-: */
/* :-: (3.B.3) private setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (3.B.4) private others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/**
 *  \brief fill time label string
 *
 *  fills given string with an ISO-conform timelabel
 *  \param pTimeLabel :- string which will be filled
 */
void LoggingUnit::fillTimeLabel(string& pTimeLabel)const
{
  char lBuffer[100];
  time_t now=time(NULL);
  tm *np=localtime(&now);
  sprintf
  ( lBuffer,"%04d%02d%02dT%02d%02d%02d",
      	    (np->tm_year+1900),(np->tm_mon+1),np->tm_mday,
      	    np->tm_hour,np->tm_min,np->tm_sec);
  pTimeLabel=lBuffer;
}  

/* --- (3.C) - protected -------------------------------------- */
/* :-: (3.C.1) protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (3.C.2) protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (3.C.3) protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/* &&& (4) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&&     methods for class LoggingSuitcase &&& */

/* --- (4.A) public ------------------------------------------ */

/* :-: (4.A.1) public constructors, inits & operators -:-:-:-: */

/**
 * \brief constructor for LoggingSuitcases without fileentries
 *  
 * Constructor for LoggingSuitcases beeing able to write messages onto the screen
 * \param pShowActionMessages :- {SHOW_NM(=0),SHOW_SM(=1),SHOW_SMADM(=2),SHOW_SMADMAHM(=3)}
 * \param pShowErrorMessages  :- {SHOW_NM(=0),SHOW_SM(=1),SHOW_SMADM(=2),SHOW_SMADMAHM(=3)}
 * 
 */
LoggingSuitcase::LoggingSuitcase
( const unsigned int pShowActionMessages,
  const unsigned int pShowErrorMessages
)
: mActionLoggingUnit(pShowActionMessages,stdout),
  mWarnLoggingUnit(pShowErrorMessages,stderr),
  mErrorLoggingUnit(pShowErrorMessages,stderr)
{
}

/**
 * \brief constructor for LoggingSuitcases  without screenmessages
 *  
 * constructor for LoggingSuitcases beeing able to write messages into a logfile
 * \param pShowActionMessages :- {SHOW_NM(=0),SHOW_SM(=1),SHOW_SMADM(=2),SHOW_SMADMAHM(=3)}
 * \param pShowErrorMessages  :- {SHOW_NM(=0),SHOW_SM(=1),SHOW_SMADM(=2),SHOW_SMADMAHM(=3)}
 * \param pLogDirPath :- (absolute) path to the directory containing the logfilespLogF
 * \param pLogFileNameKernel :- logfilenamekernel which can be expanded by _action.log etc.
 * \param pUsesSharedLoggingFile :- { USE_SHARED_LOGFILE, USE_OWN_LOGFILE }
 */
LoggingSuitcase::LoggingSuitcase
( const unsigned int pShowActionMessages,
  const unsigned int pShowErrorMessages,
  const string& pLogDirPath,
  const string& pLogFileNameKernel,
  bool pUsesSharedLoggingFile
)
: mActionLoggingUnit
  ( pShowActionMessages, 
    (pLogDirPath + "/" + pLogFileNameKernel + "_action.log"),
    pUsesSharedLoggingFile
  ),
  mWarnLoggingUnit
  ( pShowErrorMessages, 
    (pLogDirPath + "/" + pLogFileNameKernel + "_warn.log"),
    pUsesSharedLoggingFile
  ),
  mErrorLoggingUnit
  ( pShowErrorMessages, 
    (pLogDirPath + "/" + pLogFileNameKernel + "_error.log"),
    pUsesSharedLoggingFile
  )
{
}

/**
 * \brief constructor for LoggingSuitcases with file- and screenmessages
 *  
 *  LoggingSuitcase beeing able to write messages onto the screen and into a logfile
 * \param pShowActionMessages :- {SHOW_NM(=0),SHOW_SM(=1),SHOW_SMADM(=2),SHOW_SMADMAHM(=3)}
 * \param pShowErrorMessages  :- {SHOW_NM(=0),SHOW_SM(=1),SHOW_SMADM(=2),SHOW_SMADMAHM(=3)}
 * \param pWithScreenMessages :- { WITH_SCREEN_MESSAGES,WITHOUT_SCREEN_MESSAGES } 
 * \param pLogDirPath :- (absolute) path to the directory containing the logfilespLogF
 * \param pLogFileNameKernel :- logfilenamekernel which can be expanded by _action.log etc.
 * \param pUsesSharedLoggingFile :- { USE_SHARED_LOGFILE, USE_OWN_LOGFILE }
 */
LoggingSuitcase::LoggingSuitcase
( const unsigned int pShowActionMessages,
  const unsigned int pShowErrorMessages,
  bool pWithScreenMessages,
  const string& pLogDirPath,
  const string& pLogFileNameKernel,
  bool pUsesSharedLoggingFile
)
: mActionLoggingUnit
  ( pShowActionMessages, 
    stdout,
    (pLogDirPath + "/" + pLogFileNameKernel + "_action.log"),
    pUsesSharedLoggingFile
  ),
  mWarnLoggingUnit
  ( pShowErrorMessages, 
    stderr,
    (pLogDirPath + "/" + pLogFileNameKernel + "_warn.log"),
    pUsesSharedLoggingFile
  ),
  mErrorLoggingUnit
  ( pShowErrorMessages, 
    stderr,
    (pLogDirPath + "/" + pLogFileNameKernel + "_error.log"),
    pUsesSharedLoggingFile
  )
{
  if (!(pWithScreenMessages))
  {
    turnOffScreenWriting();
  }   
}
/**
 * \brief deconstructor for each type of LoggingSuitcases
 */
LoggingSuitcase::~LoggingSuitcase()
{

}
/* :-: (4.A.2) public getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (4.A.3) public setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (4.A.4) public others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/* --- (4.B) - private ---------------------------------------- */
/* :-: (4.B.1) private constructors, inits & operators -:-:-:-: */

/**
 *  \brief copy-constructor
 *
 *  class copy-constructor (mustn't be called)
 *
 *  \param co constant reference onto copyable object
 *  \return new object with duplicated content
 *
LoggingSuitcase::LoggingSuitcase(const LoggingSuitcase& co)
{ 
  assert(0);
}
*/

/* :-: (4.B.2) private getter -:-:-:-:-:-:-:-:-:-::-:-:-:-:-:-: */
/* :-: (4.B.3) private setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (4.B.4) private others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/* --- (4.C) - protected -------------------------------------- */
/* :-: (4.C.1) protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (4.C.2) protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (4.C.3) protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/* &&& (X) &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* &&&     methods for class &&& */

/* --- (X.A) - public ----------------------------------------- */
/* :-: (X.A.1) public constructors, inits & operators :-:-:-:-: */
/* :-: (X.A.2) public getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (X.A.3) public setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (X.A.4) public others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/* --- (X.B) - private ---------------------------------------- */
/* :-: (X.B.1) private constructors, inits & operators -:-:-:-: */
/* :-: (X.B.2) private getter -:-:-:-:-:-:-:-:-:-::-:-:-:-:-:-: */
/* :-: (X.B.3) private setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (X.B.4) private others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

/* --- (X.C) - protected -------------------------------------- */
/* :-: (X.C.1) protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (X.C.2) protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
/* :-: (X.C.3) protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

