/*
 * Copyright (C) 2001 Karsten Reincke <reincke@gnukose.org>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 * file <muo_logging.h> version <#1.3.1#> of project <MrProjext>
 */   

/** \file muo_logging.h
 *  \brief includable declarations of and for classes for logging methods
 *
 *  \author karsten reincke
 */

#ifndef  MUO_LOGGING_H
#define  MUO_LOGGING_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <string>
#include <stdio.h>
#include <stdlib.h>
/* &&& exported preprocessor-defines &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
#define SHOW_NO_MESSAGES 0 ///< hide all messages
#define SHOW_NM 0  ///< hide all messages
#define SHOW_SIGNALS 1	///< show only signals, no descriptions or hints
#define SHOW_SM 1 ///< show only signals, no descriptions or hints
#define SHOW_SIGNALS_AND_DESCRIPTIONS 2 ///< show signals + descriptions, but no hints
#define SHOW_SMADM 2  ///< show signals + descriptions, but no hints
#define SHOW_SIGNALS_DESCRIPTIONS_AND_HINTS 3 ///< show signals, descriptions + hints 
#define SHOW_SMADMAHM 3 ///< show signals, descriptions + hints

#define USE_SHARED_LOGFILE true ///< LoggingSuitcase shares its logfile with another
#define USE_OWN_LOGFILE false  ///< LoggingSuitcase uses its own logfile
#define WITH_SCREEN_MESSAGES true ///< LoggingSuitcase writes onto the screen
#define WITHOUT_SCREEN_MESSAGES false  ///< LoggingSuitcase writes onto the screen

#define USE_TWOLINES true ///< present the two-string-message in two lines
#define USE_ONELINE false ///< present the two-string-message in one lines
#define SIGNAL_MESSAGE 1  ///< this is an important signal message
#define SMESS 1       	  ///< this is an important signal message
#define SM 1  	      	  ///< this is an important signal message
#define DESCRIPTION_MESSAGE 2 ///< this is a describing message
#define DMESS 2       	      ///< this is a describing message
#define DM 2  	      	      ///< this is a describing message
#define HINT_MESSAGE 3  ///< this is a not very important hint (message)
#define HMESS 3       	///< this is a not very important hint (message)
#define HM 3  	      	///< this is a not very important hint (message)
/* &&& class-declarations &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */

using namespace std;

namespace krmuo
{

/**
 *  \brief class for managing logging messages
 *   
 *  instances of the class LoggingSuitcase have the task to
 *  write logging messages into a file and/or onto the screen
 */
class LoggingUnit
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  private:
    /**
     * can contain the following values: 0 = NONE , 1 = ONLY SIGNALS , 
     * 2 = SIGNALS+DESCRIPTIONS , 3 = SIGNALS+DESCRIPTIONS+HINTS 
     */
    unsigned int mMaxVisibleMessageLevel;
    /**
     * pointer to stdout or stderr or NULL
     */
    FILE *mScreen;
    /**
     * filename of the loggingfile normally including the absolute path
     */
    const string mLogFileName;
    /**
     * pointer to the opened loggingfile or NULL
     */
    FILE *mFile;
    /**
     * true if the loggingfile shall be opened just before adding
     * a logging entry and closed directly after having written the message
     * so that other LoggingUnits can use the same loggingfile without
     * being able to write into loggingfile being opened by another
     * LoggingUnit
     */
    bool mUsesSharedLoggingFile;
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for LoggingUnits without fileentries
     */
    LoggingUnit
    ( const unsigned int pMaxVisibleMessageLevel,
      FILE *pScreen);

    /** 
     *  \brief constructor for LoggingUnits without screenentries
     */    
    LoggingUnit
    ( const unsigned int pMaxVisibleMessageLevel,
      const string& pLogFileName,
      const bool pUsesSharedLoggingFile);
    
    /** 
     *  \brief constructor for LoggingUnits with screen- and fileentries
     */    
    LoggingUnit
    ( const unsigned int pMaxVisibleMessageLevel,
      FILE *pScreen, 
      const string& pLogFileName,
      const bool pUsesSharedLoggingFile);
    
    ~LoggingUnit();                      ///< class-destructor
          
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */ 
    /**
     * \brief set mScreen to Null
     *
     * for sending no longer screenmessages
     */
    inline void turnOffScreenWriting() { mScreen=NULL; };
    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /**
     * \brief write one-string-message into the LoggingUnit
     */
    void makeEntry( const string& pLogString,
      	      	    const unsigned int pLogEntryLevel) const;
    /**
     * \brief write two2one-string-message into the LoggingUnit
     */
    void makeEntry( const string& pLogString1,
      	      	    const string& pLogString2, 
      	      	    const unsigned int pLogEntryLevel) const;

    /**
     * \brief write three2one-string-message into the LoggingUnit
     */
    void makeEntry( const string& pLogString1,
      	      	    const string& pLogString2, 
		    const string& pLogString3, 
      	      	    const unsigned int pLogEntryLevel) const;

    /**
     * \brief write two2oneOrTwo-string-message into the LoggingUnit
     */
    void makeEntry( const string& pLogString1,
      	      	    const string& pLogString2,
	      	    const unsigned int pLogEntryLevel,
	      	    bool pTwoLine) const;
    /**
     * \brief write three2oneOrTwo-string-message into the LoggingUnit
     */
    void makeEntry( const string& pLogString1,
      	      	    const string& pLogString2,
		    const string& pLogString3,
	      	    const unsigned int pLogEntryLevel,
	      	    bool pTwoLine) const;
    /**
     * \brief write string-uint-2-one-string-message into the LoggingUnit
     */
    void makeEntry( const string& pLogString,
      	      	    const unsigned int pLogUInt, 
      	      	    const unsigned int pLogEntryLevel) const;
    /**
     * \brief write string-int-2-one-string-message into the LoggingUnit
     */
    void makeEntry( const string& pLogString,
      	      	    const int pLogInt, 
      	      	    const unsigned int pLogEntryLevel) const;
    
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     LoggingUnit(const LoggingUnit& co);     ///< copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /**
     *  \brief fill time label string
     */
    void fillTimeLabel(string& pTimeLabel) const;
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
  
};
/**
 *  \brief class for managing action-, warn- and errorloggingmessages
 *   
 *  instances of the class LoggingSuitcase owns three LoggingUnits,
 *  a LoggingUnit for Actionmessages, a LoggingUnit for Warnmessages
 *  and a LoggingUnit for Errormessages.
 */

class LoggingSuitcase
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  private:
    LoggingUnit mActionLoggingUnit;
    LoggingUnit mWarnLoggingUnit;
    LoggingUnit mErrorLoggingUnit;
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */

    /** 
     * \brief constructor for LoggingSuitcases without fileentries
     */
    LoggingSuitcase
    ( const unsigned int pActionMessageLevel,
      const unsigned int pErrorMessageLevel
    );                       
    
    /** 
     * \brief constructor for LoggingSuitcases  without screenmessages
     */
    LoggingSuitcase
    ( const unsigned int pShowActionMessages,
      const unsigned int pShowErrorMessages,
      const string& pLogDirPath,
      const string& pLogFileNameKernel,
      bool pUsesSharedLoggingFile
    );
        
    /**
     * \brief constructor for LoggingSuitcases with file- and screenmessages
     */
    LoggingSuitcase
    ( const unsigned int pShowActionMessages,
      const unsigned int pShowErrorMessages,
      bool pWithScreenMessages,
      const string& pLogDirPath,
      const string& pLogFileNameKernel,
      bool pUsesSharedLoggingFile
    );

    ~LoggingSuitcase();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    
    /**
     * \brief set mScreen to Null
     *
     * for sending no longer screenmessages
     */
    inline void turnOffScreenWriting() 
    { 
      mErrorLoggingUnit.turnOffScreenWriting();
      mWarnLoggingUnit.turnOffScreenWriting();
      mActionLoggingUnit.turnOffScreenWriting();
    };
    
    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    
    /**
     * \brief write one-string-message into the ActionLoggingUnit
     */
    inline
    void makeActionLogEntry
    ( const string& pLogString,
      const unsigned int pLogEntryLevel
    )const
    {
      mActionLoggingUnit.makeEntry(pLogString,pLogEntryLevel);
    }

    /**
     * \brief write two2one-string-message into the ActionLoggingUnit
     */
    inline
    void makeActionLogEntry
    ( const string& pLogString1,
      const string& pLogString2,
      const unsigned int pLogEntryLevel
    )const
    {
      mActionLoggingUnit.makeEntry(pLogString1,pLogString2,pLogEntryLevel);
    }
    
    /**
     * \brief write two2oneOrTwo-string-message into the ActionLoggingUnit
     */
    
    inline
    void makeActionLogEntry
    ( const string& pLogString1,
      const string& pLogString2,
      const unsigned int pLogEntryLevel,
      bool pTwoLine
    )const
    {
      mActionLoggingUnit.makeEntry(pLogString1,pLogString2,pLogEntryLevel,pTwoLine);
    }
    
    /**
     * \brief write string-uint-2-one-string-message into the ActionLoggingUnit
     */
    inline
    void makeActionLogEntry
    ( const string& pLogString,
      const unsigned int pLogUInt,
      const unsigned int pLogEntryLevel
    )const
    {
      mActionLoggingUnit.makeEntry(pLogString,pLogUInt,pLogEntryLevel);
    }

    /**
     * \brief write string-int-2-one-string-message into the ActionLoggingUnit
     */
    inline
    void makeActionLogEntry
    ( const string& pLogString,
      const int pLogInt,
      const unsigned int pLogEntryLevel
    )const
    {
      mActionLoggingUnit.makeEntry(pLogString,pLogInt,pLogEntryLevel);
    }

    /**
     * \brief write one-string-message into the ActionLoggingUnit
     */
    inline
    void makeWarnLogEntry
    ( const string& pLogString,
      const unsigned int pLogEntryLevel
    )const
    {
      mWarnLoggingUnit.makeEntry(pLogString,pLogEntryLevel);
    }

    /**
     * \brief write two2one-string-message into the ActionLoggingUnit
     */
    inline
    void makeWarnLogEntry
    ( const string& pLogString1,
      const string& pLogString2,
      const unsigned int pLogEntryLevel
    )const
    {
      mWarnLoggingUnit.makeEntry(pLogString1,pLogString2,pLogEntryLevel);
    }
    
    /**
     * \brief write two2oneOrTwo-string-message into the ActionLoggingUnit
     */
    
    inline
    void makeWarnLogEntry
    ( const string& pLogString1,
      const string& pLogString2,
      const unsigned int pLogEntryLevel,
      bool pTwoLine
    )const
    {
      mWarnLoggingUnit.makeEntry(pLogString1,pLogString2,pLogEntryLevel,pTwoLine);
    }
    
    /**
     * \brief write string-uint-2-one-string-message into the ActionLoggingUnit
     */
    inline
    void makeWarnLogEntry
    ( const string& pLogString,
      const unsigned int pLogUInt,
      const unsigned int pLogEntryLevel
    )const
    {
      mWarnLoggingUnit.makeEntry(pLogString,pLogUInt,pLogEntryLevel);
    }

    /**
     * \brief write string-int-2-one-string-message into the ActionLoggingUnit
     */
    inline
    void makeWarnLogEntry
    ( const string& pLogString,
      const int pLogInt,
      const unsigned int pLogEntryLevel
    )const
    {
      mWarnLoggingUnit.makeEntry(pLogString,pLogInt,pLogEntryLevel);
    }
    
    /**
     * \brief write one-string-message into the ActionLoggingUnit
     */
    inline
    void makeErrorLogEntry
    ( const string& pLogString,
      const unsigned int pLogEntryLevel
    )const
    {
      mErrorLoggingUnit.makeEntry(pLogString,pLogEntryLevel);
    }

    /**
     * \brief write two2one-string-message into the ActionLoggingUnit
     */
    inline
    void makeErrorLogEntry
    ( const string& pLogString1,
      const string& pLogString2,
      const unsigned int pLogEntryLevel
    )const
    {
      mErrorLoggingUnit.makeEntry(pLogString1,pLogString2,pLogEntryLevel);
    }
    
    /**
     * \brief write two2oneOrTwo-string-message into the ActionLoggingUnit
     */
    
    inline
    void makeErrorLogEntry
    ( const string& pLogString1,
      const string& pLogString2,
      const unsigned int pLogEntryLevel,
      bool pTwoLine
    )const
    {
      mErrorLoggingUnit.makeEntry(pLogString1,pLogString2,pLogEntryLevel,pTwoLine);
    }
    
    /**
     * \brief write string-uint-2-one-string-message into the ActionLoggingUnit
     */
    inline
    void makeErrorLogEntry
    ( const string& pLogString,
      const unsigned int pLogUInt,
      const unsigned int pLogEntryLevel
    )const
    {
      mErrorLoggingUnit.makeEntry(pLogString,pLogUInt,pLogEntryLevel);
    }

    /**
     * \brief write string-int-2-one-string-message into the ActionLoggingUnit
     */
    inline
    void makeErrorLogEntry
    ( const string& pLogString,
      const int pLogInt,
      const unsigned int pLogEntryLevel
    )const
    {
      mErrorLoggingUnit.makeEntry(pLogString,pLogInt,pLogEntryLevel);
    }
    
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     LoggingSuitcase(const LoggingSuitcase& co);     ///< copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
  
};

}
#endif
