/*
 * Copyright (C) 2001 Karsten Reincke <karsten.reincke@karubik.de>
 *  
 *  
 * This file is part of of the software-project MRPROJEXT.
 * 
 * MRPROJEXT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * MRPROJEXT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 * file <mrprjContent.h> version <#1.3.1#> of project <MrProjext>
 */   

/** \file mrprjContent.h
 *  \brief includable declarations of and for the project MrProjext
 *
 *  \author karsten reincke
 */

#ifndef  MRPRJCONTENT_H
#define  MRPRJCONTENT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <map>
#include <list>
#include <muo.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

/* &&& exported preprocessor-defines &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */

/* &&& class-declarations &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */

namespace mrprj
{

/**
 * \brief object for converting timesstrings (units) into other timestrings (units)
 *
 */

class DateStringConverter
{
  private:
  public:
    
    DateStringConverter();  ///< constructor for the class DateStringConverter
    ~DateStringConverter(); ///< destructor for the class DateStringConverter

    /**
     *  \brief vor converting a Mrproject-Timestring into a Yank-Timestring
     */    
    string formYankDateString(const string& pMrprojectDateString)const;
    
    /**
     *  \brief vor converting a Mrproject-Timestring into an abbreviated german timestring
     */     
   string formGermanDateString(const string& pMrprojectDateString)const;

};

/**
 * \brief object for getting contents of xmlNodes as human-readable strings
 *
 *  Instances of this class are able to read the utf8-encoded strings
 *  of an xmlNode (of the libxml) and to covert those strings to
 *  iso-latin-1-strings
 */
class XmlNodeReader
{
  private:
    /**
     *  \brief the xmlConverter
     * 
     *  for decoding and encoding xmlStrings -> isoLatin1Strings
     */
    krmuo::XmlConverter xmlConverter;  

  public:
    XmlNodeReader();    ///< constructor for the class XmlNodeReader
    
    ~XmlNodeReader();  ///< destructor for the class XmlNodeReader


    /**
     *  \brief for reading an xmlNode-property if it's defined
     */
    bool getStringProperty
    ( 
      const xmlDocPtr xmlDoc,
      const xmlNodePtr pPropertyNode, 
      const string& pPropertySigner,
      string& pPropertyValue
    );

    
    /**
     *  \brief for reading an int-xmlNode-property if it's defined inside of the xmlNode
     *
     *  using the method getStringProperty this method returns
     *  the property-content as int-value
     *  \param xmlDoc :- the dom of the mrprj-file, needed by the pecial libxml-function
     *  \param pPropertyNode :- node whose property-value shall be got
     *  \param pPropertySigner :- determiner of the property
     *  \param pPropertyValue :- the decoded and converted int-value
     *
     *  \return true if the attribute is defined inside of the xml-node otherwise false
     */
    
    inline bool getIntProperty
    ( 
      const xmlDocPtr xmlDoc,
      const xmlNodePtr pPropertyNode, 
      const string& pPropertySigner,
      int& pPropertyValue
    )
    {
      string localStringValue;
      if (!(getStringProperty(xmlDoc,pPropertyNode,pPropertySigner,localStringValue)))
        return false;
      pPropertyValue=atoi(localStringValue.c_str());
      return true;
    }
    
    /**
     *  \brief for reading a long-xmlNode-property if it's defined inside of the xmlNode
     *
     *  using the method getStringProperty this method returns
     *  the property-content as int-value
     *  \param xmlDoc :- the dom of the mrprj-file, needed by the pecial libxml-function
     *  \param pPropertyNode :- node whose property-value shall be got
     *  \param pPropertySigner :- determiner of the property
     *  \param pPropertyValue :- the decoded and converted int-value
     *
     *  \return true if the attribute is defined inside of the xml-node otherwise false
     */
    
    inline bool getLongProperty
    ( 
      const xmlDocPtr xmlDoc,
      const xmlNodePtr pPropertyNode, 
      const string& pPropertySigner,
      long& pPropertyValue
    )
    {
      string localStringValue;
      if (!(getStringProperty(xmlDoc,pPropertyNode,pPropertySigner,localStringValue)))
        return false;
      pPropertyValue=atol(localStringValue.c_str());
      return true;
    }


    /**
     *  \brief for reading a float-xmlNode-property if it's defined inside of the xmlNode
     *
     *  using the method getStringProperty this method returns
     *  the property-content as int-value
     *  \param xmlDoc :- the dom of the mrprj-file, needed by the pecial libxml-function
     *  \param pPropertyNode :- node whose property-value shall be got
     *  \param pPropertySigner :- determiner of the property
     *  \param pPropertyValue :- the decoded and converted int-value
     *
     *  \return true if the attribute is defined inside of the xml-node otherwise false
     */
    
    inline bool getFloatProperty
    ( 
      const xmlDocPtr xmlDoc,
      const xmlNodePtr pPropertyNode, 
      const string& pPropertySigner,
      float& pPropertyValue
    )
    {
      string localStringValue;
      if (!(getStringProperty(xmlDoc,pPropertyNode,pPropertySigner,localStringValue)))
        return false;
      pPropertyValue=atof(localStringValue.c_str());
      return true;
    }

};

/**
 *  \brief for managing the content of the class Property
 *   
 *  instances of the class Property have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a property is defined
 */
class Property
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    krmuo::LoggingUnit& mLoggingUnit;///< for being able to print fault messages to stderr
    string mName; ///< the name of the property
    string mType; ///< the type of the property
    string mOwner; ///< the owner of the property
    string mLabel; ///< the owner of the property
    string mDescription; ///< the owner of the property
    
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class Property
     */
    Property
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& pName,
      const string& pType,
      const string& pOwner,
      const string& pLabel,
      const string& pDescription
    );                       
        
    ~Property();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

    /**
     *   \brief returns the content of mName
     */
    inline const string& getName() const { return mName; }

    /**
     *   \brief returns the content of mType 
     */
    inline const string& getType() const { return mType; }

    /**
     *   \brief returns the content of mOwne 
     */
    inline const string& getOwner() const { return mOwner; }

    /**
     *   \brief returns the content of mLabel 
     */
    inline const string& getLabel() const { return mLabel; }

    /**
     *   \brief returns the content of mDescription 
     */
    inline const string& getDescription() const { return mDescription; }


    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     Property(const Property& co);     ///< unusable copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};

/**
 *  \brief for managing the content of the class Phase
 *   
 *  instances of the class Phase have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a phase is defined
 */
class Phase
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    krmuo::LoggingUnit& mLoggingUnit;///< for being able to print fault messages to stderr
    string mName; ///< the name of the phase
    
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class Phase
     */
    Phase
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& pName
    );                       
        
    ~Phase();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

    /**
     *   \brief returns content of mName
     */
    inline const string& getName() const { return mName; }


    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     Phase(const Phase& co);     ///< unusable copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};

/**
 *  \brief for managing the content of the class DayType
 *   
 *  instances of the class DayType have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a DayType is defined
 */
class DayType
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    krmuo::LoggingUnit& mLoggingUnit;///< for being able to print fault messages to stderr
    string mId; ///< the id of the DayType
    string mName; ///< the name of the DayType
    string mDescription; ///< the description of the DayType
    
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class DayType
     */
    DayType
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& pId,
      const string& pName,
      const string& pDescription
    );                       
        
    ~DayType();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

    /**
     *   \brief returns content of mId
     */
    inline const string& getId() const { return mId; }

    /**
     *   \brief returns content of mName
     */
    inline const string& getName() const { return mName; }


   /**
     *   \brief returns content of mDescription
     */
    inline const string& getDescription() const { return mDescription; }


    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     DayType(const DayType& co);     ///< usable!!! copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};

/**
 *  \brief for managing the content of the class Interval
 *   
 *  instances of the class Interval have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a Interval is defined
 */
class Interval
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    krmuo::LoggingUnit& mLoggingUnit;///< for being able to print fault messages to stderr
    string mStart; ///< the starttime of the interval
    string mEnd; ///< the endtime of the interval
    
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class Interval
     */
    Interval
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& pStart,
      const string& pEnd
    );                       
        
    ~Interval();                      ///< class-destructor
        
     Interval(const Interval& co);     ///< !!! usable copy-constructor
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

    /**
     *   \brief returns content of mMon(day)-Day-type
     */
    inline const string& getStart() const { return mStart; }

    /**
     *   \brief returns content of mTues(day)-Day-type
     */
    inline const string& getEnd() const { return mEnd; }


    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};


/**
 *  \brief for managing the content of the class OverriddenDayType
 *   
 *  instances of the class OverriddenDayType have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a OverriddenDayType is defined
 */
class OverriddenDayType
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
    list<Interval> mIntervalList;    
  
  private:
    krmuo::LoggingUnit& mLoggingUnit;///< for being able to print fault messages to stderr
    string mId; ///< the id of the OverriddenDayType
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class OverriddenDayType
     */
    OverriddenDayType
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& pId
    );                       
        
    ~OverriddenDayType();                      ///< class-destructor
        

    OverriddenDayType(const OverriddenDayType& co);     ///< usable copy-constructor
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

    /**
     *   \brief returns content of mId
     */
    inline const string& getId() const { return mId; }


    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};

/**
 *  \brief for managing the content of the class Day
 *   
 *  instances of the class Day have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a day is defined
 */
class Day
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    krmuo::LoggingUnit& mLoggingUnit;///< for being able to print fault messages to stderr
    string mDate; ///< the date of the day
    string mType; ///< the type of the day
    string mId; ///< the id of the day
    
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class Day
     */
    Day
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& pDate,
      const string& pType,
      const string& pId
    );                       
        
    ~Day();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

    /**
     *   \brief returns content of mDate
     */
    inline const string& getDate() const { return mDate; }

    /**
     *   \brief returns content of mType
     */
    inline const string& getType() const { return mType; }

    /**
     *   \brief returns content of mId
     */
    inline const string& getId() const { return mId; }


    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
     Day(const Day& co);     ///< usable copy-constructor
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};

/**
 *  \brief for managing the content of the class Calendar
 *   
 *  instances of the class Calendar have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a calendar is defined
 */
class Calendar
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
    string mMon; ///< the id of the day-type of Monday
    string mTue; ///< the id of the day-type of Tuesday
    string mWed; ///< the id of the day-type of Wednesday
    string mThu; ///< the id of the day-type of Thursday
    string mFri; ///< the id of the day-type of Friday
    string mSat; ///< the id of the day-type of Saturday
    string mSun; ///< the id of the day-type of Sunday
    
    /**
     *  List of overridden day-types
     */
    list<OverriddenDayType> mOverriddenDayTypeList;
    /**
     * List of day definitions /specifications
     */
    list<Day> mDayList;  
  private:
    krmuo::LoggingUnit& mLoggingUnit;///< for being able to print fault messages to stderr
    string mId; ///< the id of the calendar
    string mName; ///< the name of the calendar
            
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class Calendar
     */
    Calendar
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& pId,
      const string& pName
    );                       
        
    ~Calendar();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

    /**
     *   \brief returns content of mId
     */
    inline const string& getId() const { return mId; }

    /**
     *   \brief returns content of mName
     */
    inline const string& getName() const { return mName; }


    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     Calendar(const Calendar& co);     ///< unusable copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};

/**
 *  \brief for managing the content of the class ResourceGroup
 *   
 *  instances of the class ResourceGroup have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a ResourceGroup
 *  of the project is described
 *
 */
class ResourceGroup
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    string mGroupId; ///< the tag-id of the described resource-group
    string mGroupTitle; ///< the title / name of the resource-group
    string mGroupDescription; ///< a resource-group description still unused by mr-project-xml-files
    string mGroupAdminName; ///< the name of the resource-group-admin ...
    string mGroupAdminPhone; ///< ... his phone-number ...
    string mGroupAdminEmail; ///< ... and his email address
    krmuo::LoggingUnit& mLoggingUnit; ///< for being able to print fault messages to stderr
    
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class ResourceGroup
     */
    ResourceGroup
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& pGroupId,
      const string& pGroupTitle="???",
      const string& pGroupDescription="???",
      const string& pGroupAdminName="???",
      const string& pGroupAdminPhone="???",
      const string& pGroupAdminEmail="???"
    );                       
        
    ~ResourceGroup();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /**
     *   \brief returns the id of the denoted resource-group
     */
    inline const string& getGroupId() const { return mGroupId; }
    
    /**
     *   \brief returns the name/title of the denoted resource-group
     */
    inline const string& getGroupTitle() const { return mGroupTitle; }
    
    /**
     *   \brief returns the name of the admin of denoted resource-group
     */
    inline const string& getGroupAdminName() const { return mGroupAdminName; }

    /**
     *   \brief returns the phone of the admin of denoted resource-group
     */
    inline const string& getGroupAdminPhone() const { return mGroupAdminPhone; }

    /**
     *   \brief returns the email-address of the admin of denoted resource-group
     */
    inline const string& getGroupAdminEmail() const { return mGroupAdminEmail; }
    
    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     ResourceGroup(const ResourceGroup& co);     ///< unusable copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};

/**
 *  \brief for managing the content of the class Resource
 *   
 *  instances of the class Resource have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a Resource
 *  of the project is described
 *
 */
class Resource
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    string mResourceId;///< the tag-id of the described resource
    string mResourceGroupId;///< the tag-id of the corresponding resourceGroup
    string mResourceTitle;///< the title / name of the resource
    string mResourceDescription;///< a resource description still unused by mr-project-xml-files
    int mResourceType; ///< the type of the resource ...
    long mResourceUnits; ///< ... its credits ...
    string mRecourceEmail; ///< ...its email address
    float mRecourceStdRate; ///< ... its std-rate ...
    float mRecourceOvtRate; ///< ... its ovt-rate ...
    krmuo::LoggingUnit& mLoggingUnit; ///< for being able to print fault messages to stderr
    
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class Resource
     */
    Resource
    ( 
      krmuo::LoggingUnit& mLoggingUnit,    
      const string& pResourceId,
      const string& pResourceGroupId,      
      const string& pResourceTitle="???",
      const string& pResourceDescription="???",
      const int pResourceType=1,
      const long pResourceUnits=100,
      const string& pRecourceEmail="???",
      float pRecourceStdRate=0.0,
      float pRecourceOvtRate=200.0
    );                       
        
    ~Resource();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /**
     *  \brief returns the resource-id
     */
    inline const string& getResourceId() const { return mResourceId; }
    
    /**
     *  \brief returns the id of the corresponding group-id
     */
    inline const string& getResourceGroupId() const { return mResourceGroupId; }

    /**
     *  \brief returns the name / title of the resource
     */
    inline const string& getResourceTitle() const { return mResourceTitle; }

    /**
     *  \brief returns the type of the resource
     */
    inline const int getResourceType() const { return mResourceType; }

    /**
     *  \brief returns the units of the resource
     */
    inline const long getResourceUnits() const { return mResourceUnits; }

    /**
     *  \brief returns the email-address of the resource
     */
    inline const string& getRecourceEmail() const { return mRecourceEmail; }

    /**
     *  \brief returns the std-rate of the resource
     */
    inline const float getRecourceStdRate() const { return mRecourceStdRate; }

    /**
     *  \brief returns the ovt-rate of the resource
     */
    inline const float getRecourceOvtRate() const { return mRecourceOvtRate; }
    
    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     Resource(const Resource& co);     ///< unusable copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};

/**
 *  \brief for managing the content of the class Allocation
 *   
 *  instances of the class Allocation have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which a task is mapped
 *  onto a resource
 */
class Allocation
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    string mTaskId; ///< the id of the denoted task of the task/resource-pair
    string mResourceId; ///< the id of the denoted resource of the task/resource-pair
    krmuo::LoggingUnit& mLoggingUnit;///< for being able to print fault messages to stderr
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class Allocation
     */
    Allocation
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& pTaskId,
      const string& pResourceId      
    );                       
        
    ~Allocation();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

    /**
     *   \brief returns the id of the task of the task/resource-pair
     */
    inline const string& getTaskId() const { return mTaskId; }

    /**
     *   \brief returns the id of the resource of the task/resource-pair
     */
    inline const string& getResourceId() const { return mResourceId; }
    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
              
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     Allocation(const Allocation& co);     ///< unusable copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};




/**
 *  \brief for managing the content of the class predecessor
 *   
 *  instances of the class Predecessor have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which the predecessors
 *  of the project-task are determined
 */
class Predecessor
{
  public:
  
  private:
    const string mEntryId;  ///< embedded number of the predecessor-tag
    const string mPredecessorTaskId;  ///< embedded number of the task being a predecessor
    const string mType;   ///< embedded type of the predecessor-tag 
  protected:

  public:
  
    /** 
     *  \brief constructor for class Predecessor
     */
    Predecessor
    ( 
      const string& pEntryId,
      const string& pPredecessorTaskId,
      const string& pType  
    );
    
    /** 
     *  \brief destructor for class Predecessor
     */
    ~Predecessor();
    
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    
    /**
     *  \brief returns the id of the predecessor-tag itself
     */
    inline const string& getEntryId() const { return mEntryId; }
    /**
     *  \brief returns the id of the denoted task being a predecessor
     */
    inline const string& getPredecessorTaskId() const { return mPredecessorTaskId; }
    /**
     *  \brief returns the type of the predecessor-tag itself
     */
    inline const string& getType() const { return mType ; }
};

/**
 *  \brief for managing the content of the class task
 *   
 *  instances of the class Task have the task to
 *  hold and change the content of those entries of
 *  the mrProject-Xml-File, by which the tasks
 *  of the project are described
 *
 *  Note: 
 *
 *  (1) each task is represented as a tree consting of the task itself
 *  and a (possibly empty) list of its subtasks.
 *
 *  (2) the main / top task is a virtual task containing the id -1
 *  and is not(!!!) a member of the original mrproject-xml-file
 *
 *  (3) each subtask will be created by the new-operator and must
 *  therefore explicitely be deleted the destructor of the (mother) task
 */

class Task
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    string mTaskId; ///< the tag-id of the described task
    string mTaskTitle; ///< the title / name of the task
    string mTaskDescription; ///< a task description still not used by mr-project-xml-files
    string mTaskStartDate; ///< start-date of the task iso-8601 encoded (yyyymmddThhmmss)
    string mTaskEndDate; ///< end-date of the task iso-8601 encoded (yyyymmddThhmmss)
    int mFullFilledPercent; ///< how many percent of the task are already fullfilled
    bool mIsEmptyTopTask;
     
    map<string, Task*> mSubTaskMap; ///< for representing the row of subtasks of this task
    list<string> mSubTaskOrderList; ///< for storing the read order of the subtask-row
    map<string,Predecessor*> mPredecessorMap; ///< for storing the predecessors of this task
    krmuo::LoggingUnit& mLoggingUnit; ///< for being able to print fault messages to stderr

    bool mIsActive; ///< for storing the state of this task after an extracting operation
    bool mIsAllocated; ///< for noting that any resource handles this task
    bool mAnyDaughterIsActive; ///< for noting that any resource handles this task
    int mXrowspan;
    int mXrowTacker;
    int mNodeEmbeddedLevel;
    bool mXdoPrint;
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    Task ///< class-constructor
    ( 
      krmuo::LoggingUnit& pLoggingUnit,  
      const string& spTaskId,
      const string& pTaskTitle="???",
      const string& pTaskDescription="???",
      const string& pTaskStartDate="19700101T000000",
      const string& pTaskEndDate="21001231T240000",
      const int pFullFilledPercent=0,
      const bool pIsEmptyTopTask=false
    );                       
        
    ~Task();                      ///< class-destructor
        
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /**
     *   \brief returns the task-id as long-value
     */
    inline const string& getTaskId() const { return mTaskId; }

    /**
     *   \brief returns the task-title (name) as string
     */
    inline const string& getTaskTitle() const { return mTaskTitle; }

    /**
     *   \brief returns the task-description (note) as string
     */
    inline const string& getTaskDescription() const { return mTaskDescription; }


    /**
     *   \brief returns the task-description (note) as string without any lf,cr,tabs etc.
     */
    const string getTaskDescriptionAsOneLine() const;

    /**
     * \brief makes comma seperated string of all predecessor ids
     */
    const string  getPredecessorString()const;

    /**
     *   \brief returns the task-description (note) as formatted string
     */
    const string getTaskDescriptionAsFormattedString
      ( const string& pIndent,
      	const int pLineLength) const;

    /**
     *   \brief returns the task-description (note) as online formatted without ",
     */
    const string getClearTaskDescriptionAsOneLine
      () const;
	
    /**
     *   \brief returns the iso-8601 encoded start-date
     */
    inline const string& getStartDate() const { return mTaskStartDate; }

    /**
     *   \brief returns the iso-8601 encoded end-date
     */
    inline const string& getEndDate() const { return mTaskEndDate; }

    /**
     *   \brief returns degree of fullfillness
     */
    inline const int getFullFilledPercent() const { return mFullFilledPercent; }


    /**
     *   \return true if this is the empty top-taks otherwise false
     */
    inline const bool isEmptyTopTask()const { return  mIsEmptyTopTask; }
    
    /**
     *   \brief returns the state of the task after an extraction
     */
    inline const bool isActive() const { return mIsActive; }

    /**
     *   \brief returns true if any resource handles this task
     */
    inline const bool isAllocated() const { return mIsAllocated; }

    /**
     *   \brief returns true if any daughter is active
     */
    inline const bool isAnyDaughterIsActive() const { return mAnyDaughterIsActive; }


    /**
     *   \brief returns the number of daughters
     */
    inline const long numberOfDaughters() const { return mSubTaskMap.size(); }
    
    
    /**
     *   \brief returns true if the number of daughters is zero
     */
    inline const bool isLeafNode() const 
    {
      if (numberOfDaughters()==0)
        return true;
      else
        return false;
    }
    
    /**
     *  \brief returns true if any part of the task still must be done
     */
     inline const bool isOpenTask() const
     { 
     	if (mFullFilledPercent==100)
      	return false;
	    else
	      return true;
     }
     
    /**
     *  \brief returns clear-text version of the done-State
     */
     inline const string getDoneDescription() const 
     { 
      	if (mFullFilledPercent==100)
      	  return "done";
	      else
	        return "open";
     }
     
    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    
    /**
     *   \brief sets the task onto the state ignorable
     */
    inline void setPassive() { mIsActive=false; }
    
    /**
     *   \brief sets the task onto the state un-ignorable
     */
    inline void setActive() { mIsActive=true; }
    /**
     *   \brief notes that any resource handles this task
     */
    inline void setAllocated() { mIsAllocated=true; }

    /**
     *   \brief notes that any resource handles this task
     */
    inline void setAnyDaughterIsActive() { mAnyDaughterIsActive=true; }
    
    /**
     *   \brief sets the task onto the state ignorable if all daughters can be ignored
     */
    void setPassiveIfAllDaughtersPassive();
    
    /**
     *  \brief updates the start-date of the task
     *
     *  \param pStartDate :- new iso-8601 encoded start-date
     */
    inline void setStartDate(const string& pStartDate) { mTaskStartDate=pStartDate; }


    /**
     *  \brief updates the end-date of the task
     *
     *  \param pEndDate :- new iso-8601 encoded end-date
     */
    inline void setEndDate(const string& pEndDate) { mTaskEndDate=pEndDate; }
    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    
    /**
     *  \brief for (recursively) transposing a task-xmlNode into the internal representation
     */  
    bool fillTaskSubStructureByMrPrjXmlTaskNodes
    ( 
      xmlDocPtr xmlDoc,
      const xmlNodePtr pFirstTaskSubnode,
      map<string,Task*>& mGlobalTaskMap
    );
        
    /**
     *  \brief for making a task ignorable if it doesn't belong to the given timeslot
     */
    bool limitToTimeSlot
    (
      const string& pTimeSlotStart,
      const string& pTimeSlotEnd
    );

    /**
     *  \brief for recursively making a task ignorable if it is already fullfiled
     */

    bool limitToOpenTasks();

    /**
     *  \brief for recursively determining the array position of the task
     */

    int insertXYposition
    ( 
      const int pEmbeddedLevel,
      int& pMaxEmbeddedLevel
    );


    /**
     *  \brief for rewriting the internal representation as mrproject-xml-file-member
     */
    bool printTaskAsMrPrjXmlFileMember(int pBlankIndents)const;
    
    /**
     *  \brief for rewriting the internal representation as leaf-memo-list-member
     */
    bool printTaskAsLeafMemoListMember
    ( const string& pTaskPrefix,
      const string& pLevelOrderString,
      const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList,
      const bool iso8601
    )const;

    /**
     *  \brief for rewriting the internal representation as tree-memo-member
     */
    bool printTaskAsTreeMemoMember
    ( const string& pLevelOrderString,
      int pTreeDepth,
      const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList,
      const bool iso8601
    )const;

    /**
     *  \brief for rewriting the internal representation as leaf-vtodo-member
     */
    bool printTaskAsLeafVTodoMember
    ( const map<string,Resource*>& pResourceMap,
     const list<Allocation*>& pAllocationList,
      const string& pConnectedProjectName
    ) const;
     
    /**
     *  \brief for rewriting the internal representation as tree-vtodo-member
     */
     bool printTaskAsTreeVTodoMember
    ( const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList,
      const string& pConnectedProjectName
    )const;
 
     
    /**
     *  \brief for rewriting the internal representation as leaf-todo-member
     */
    bool printTaskAsPilotLeafTodoMember
    ( const string& pLevelOrderString,
      const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList
    )const;

    /**
     *  \brief for rewriting the internal representation as tree-todo-member
     */
    bool printTaskAsPilotTreeTodoMember
    ( const string& pLevelOrderString,
      const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList
    )const;

    /**
     *  \brief for rewriting the internal representation as leaf-yank-todo-member
     */
    bool printTaskAsLeafYankTodoMember
    ( const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList
    )const;

    /**
     *  \brief for rewriting the internal representation as tree-yank-todo-member
     */
    bool printTaskAsTreeYankTodoMember
    ( const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList,
      const int pBlankIndents
    )const;
   
   /**
    *  \brief for rewriting the internal representation as leaf-cls-todo-member
    */
    bool printTaskAsLeafCslTodoMember
   ( const string& pLevelOrderString,
     const map<string,Resource*>& pResourceMap,
     const list<Allocation*>& pAllocationList
   )const;
   
   
   /**
    *  \brief for rewriting the internal representation as tree-csl-todo-member
    */
    bool printTaskAsTreeCslTodoMember
    ( const string& pLevelOrderString,
      const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList
    )const;


  /**
   *  \brief for rewriting the internal representation as set of html-table-rows
   */
    bool printTaskAsHtmlSurveyMember
    ( const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList,
      const int maxPathLength,
      const string& pSurveyReferenceDayString     
    );

  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     Task(const Task& co);     ///< copy-constructor (undefined -> unusable)
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /**
     * \brief insert a new subTask into this->mSubTaskMap
     */
    Task* insertSubTask 
    ( 
      const string& pSubTaskId,
      const string& pSubTaskTitle="???",
      const string& pSubTaskDescription="???",
      const string& pSubTaskStartDate="19700101T000000",
      const string& pSubTaskEndDate="21001231T240000",
      const int pFullFilledPercent=0      
    );
    
    /**
     * \brief for filling the predecessor-map of this tasks
     */
    bool fillTaskPredecessorByMrPrjXmlPredecessorNodes
    ( 
      xmlDocPtr xmlDoc,
      const xmlNodePtr pFirstPredecessors
    );

    /**
     *  \brief for searching the resource-infos belonging to this task
     */
    bool getResourceInfos
    (
      const map<string,Resource*>& pResourceMap,
      const list<Allocation*>& pAllocationList,
      string& pResourceId,
      string& pResourceTitle
    ) const;
      
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};



/**
 *  \brief for managing the content of the class TaskField
 *   
 *  instances of the class TaskField have the task to
 *  hold, change and reqrite the content of all entries of
 *  the mrProject-Xml-File,
 */
class TaskField
{
  /* &&& attributes &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  public:
  
  private:
    
  
    Task mMainTask; ///< main task containing all tasks of the mrprj-xml-file as subtasks
    map<string,Task*> mGlobalTaskMap;  ///< flat task-map containing all task-ids and their pointers
      
    map<string,ResourceGroup*> mResourceGroupMap; ///< flat resource-group-map
    string mDefaultResourceGroup;
    
    map<string,Resource*> mResourceMap; ///< flat resource-map
    
    list<Allocation*> mAllocationList; ///< list of allocated allocations for mapping tasks to resources
    list<Property*> mPropertyList; ///< list of allocated properties
    list<Phase*> mPhaseList; ///< list of allocated phases
    list<DayType*> mDayTypeList; ///< list of allocated phases
    list<Calendar*> mCalendarList; ///< list of allocated phases
    
    krmuo::LoggingUnit& mLoggingUnit; ///< reference to the use fault-message-writer
    
    string mProjectName;
    string mProjectCompany;
    string mProjectManager;
    string mProjectPhase;
    string mProjectStart;
    string mProjectMrpVersion;
    string mProjectCalendar;
    
    string mRealTimeslotStart;
    string mRealTimeslotEnd;
  protected:
  
  /* &&& methods &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
  
  public:
  
    /* :-: public constructors / init / operators :-:-:-:-:-:-:-: */
    
    /** 
     *  \brief constructor for class TaskField
     */
    TaskField(krmuo::LoggingUnit& pLoggingUnit);                       
        
    ~TaskField();                      ///< class-destructor
        
  
    /* :-: public getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: public setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */    
    /* :-: public others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /**
     *  \brief for filling the whole TaskField
     */
    bool fillTaskFieldByMrPrjXmlFileContent
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      const string& mrprjBaseFileContent
    );
        
    /**
     *  \brief for making all those tasks of the field ignorable which doesn't belong to the given timeslot
     *  
     *  \param pTimeSlotStart :- iso-8601 encoded timeslot-start-date
     *  \param pTimeSlotEnd :- iso-8601 encoded timeslot-end-date
     *
     */
    inline void limitToTimeSlot(const string& pTimeSlotStart,const string& pTimeSlotEnd)
    {
      mMainTask.limitToTimeSlot(pTimeSlotStart,pTimeSlotEnd);
    }

    /**
     *  \brief for making all those tasks of the field ignorable which are already fullfilled
     */
    inline void limitToOpenTasks()
    {
      mMainTask.limitToOpenTasks();
    }
   
   
    /**
     *  \brief set those tasks as ignorable which  won't be fullfilled by the given resource
     */
    bool limitToResource(const string& pResourceTitel);
    
    /**
     *  \brief set those tasks as ignorable which won't be fullfilled by users of the given group
     */
    bool limitToResourceGroup(const string& pResourceGroupTitel);

    /**
     *  \brief sets all tasks onto the state ignorable if all daughters can be ignored
     */
    inline void limitToThoseWithActiveDaughters()
    {
      mMainTask.setPassiveIfAllDaughtersPassive();
    }
    
    /**
     *  \brief for rewriting the whole TaskField to stdout using the mrprj-xml-file-style
     */
    
    bool printTaskFieldAsMrPrjXmlFile()const;
    
    
   /**
     *  \brief for rewriting the whole TaskField to stdout using a leaf oriented list style
     */
    
    bool printTaskFieldAsLeafListMemo(bool pIso8601);

   /**
     *  \brief for rewriting the whole TaskField to stdout using a tree oriented list style
     */
    
    bool  printTaskFieldAsTreeMemo(bool pIso8601);
    
    /**
     *  \brief for rewriting all active nodes/tasks as pilot-todo-list
     */
    inline void printTaskFieldAsPilotTreeTodo()
    {
      mMainTask.printTaskAsPilotTreeTodoMember("",mResourceMap,mAllocationList);

    }
    
    /**
     *  \brief for rewriting all active leaf-nodes/tasks as pilot-todo-list
     */
    inline void printTaskFieldAsPilotLeafTodo()
    {
      mMainTask.printTaskAsPilotLeafTodoMember("",mResourceMap,mAllocationList);

    }

    /**
     *  \brief for rewriting all active leaf-nodes/tasks as vcrd-file-part
     */
    inline void printTaskFieldAsLeafVTodo()
    {
      mMainTask.printTaskAsLeafVTodoMember
      	(mResourceMap,mAllocationList,getConnectProjectName());

    }
    /**
     *  \brief for rewriting all active nodes/tasks as vcrd-file-part
     */
    inline void printTaskFieldAsTreeVTodo()
    {
      mMainTask.printTaskAsTreeVTodoMember
      	(mResourceMap,mAllocationList,getConnectProjectName());

    }
    /**
     *  \brief for rewriting all active leaf-nodes/tasks as yank-file
     */
    void printTaskFieldAsLeafYankTodo();
    
    /**
     *  \brief for rewriting all active leaf-nodes/tasks as yank-file
     */
    void printTaskFieldAsTreeYankTodo();
    
    
    /**
     *  for rewriting all active leaf-nodes/tasks as commaseperated list (gnumeric/excel)
     */
    void printTaskFieldAsLeafCslTodo();

   /**
     *  for rewriting all active nodes/tasks as commaseperated list (gnumeric/excel)
     */
     void printTaskFieldAsTreeCslTodo();

    /**
     *  for rewriting all active nodes/tasks as commaseperated list (gnumeric/excel)
     */
     void printTaskHtmlSurvey(const string& pSurveyReferenceDayString);
 
  private:
    /* :-: private constructors / init / operators :-:-:-:-:-:-: */
     TaskField(const TaskField& co);     ///< unusable copy-constructor
    /* :-: private getter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    string getConnectProjectName()const;
    /* :-: private setter -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: private others -:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */

    /**
     *  \brief for transposing all resource-Group-xmlNodes into the internal representation
     */  
    bool fillResourceGroupMapByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc,
      const xmlNodePtr pFirstResourceGroup
    );

    /**
     *  \brief for transposing all resource-xmlNodes into the internal representation
     */  
    bool fillResourceMapByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstResource
    );

    /**
     *  \brief for transposing all allocation-xmlNodes into the internal representation
     */  
    bool fillAllocationListByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstAllocation
    );
    
    /**
     *  \brief for transposing all allocation-xmlNodes into the internal representation
     */  
    bool fillPropertyListByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstProperty
    );
    
    /**
     *  \brief for transposing all allocation-xmlNodes into the internal representation
     */  
    bool fillPhaseListByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstProperty
    );
    
    
    /**
     *  \brief for transposing all allocation-xmlNodes into the internal representation
     */  
    bool fillDayTypeListByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstProperty
    );

    bool fillIntervalListByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstInterval,
      list<Interval>& pIntervalList
    );


    bool fillOverriddenDayTypeListByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstOverriddenDayType,
     list<OverriddenDayType>& pOverriddenDayTypeList
    );


    bool fillDayListByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstDay,
      list<Day>& pDayList
    );



    bool fillCalSubPartByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstCalSubPart,
      Calendar* pCalendar
    );


    /**
     *  \brief for transposing all allocation-xmlNodes into the internal representation
     */  
    bool fillCalendarListByMrPrjXmlEntries
    ( 
      krmuo::LoggingUnit& pLoggingUnit,
      xmlDocPtr xmlDoc, 
      const xmlNodePtr pFirstProperty
    );


    /**
     * \brief for getting the really used min and max of start- and end-date
     */
    void storeRealTimeSlot();
    
  protected:
    /* :-: protected getter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected setter :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
    /* :-: protected others :-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-:-: */
};
}
#endif
